@@ -1,7 +1,7 @@
 /*
  * Miscellaneous routines for Mathomatic.
  *
- * Copyright (C) 1987-2008 George Gesslein II.
+ * Copyright (C) 1987-2009 George Gesslein II.
  */
 
 #include "includes.h"
@@ -17,7 +17,7 @@ const char	*str;
 #if	!SILENT
 	set_color(2);		/* set color to red */
 	printf("%s\n", str);
-	default_color();
+	default_color();	/* restore to default color */
 #endif
 }
 
@@ -26,8 +26,9 @@ const char	*str;
  */
 void
 warning(str)
-char	*str;
+const char	*str;
 {
+	warning_str = str;	/* must be a constant string, temporary strings don't work */
 #if	!SILENT
 	if (debug_level >= 0) {
 		set_color(1);	/* set color to yellow */
@@ -48,8 +49,23 @@ error_huge()
 	longjmp(jmp_save, 14);
 }
 
+/*
+ * This is called when a bug test result is positive.
+ *
+ * There is no return.
+ */
+void
+error_bug(str)
+const char	*str;	/* constant string to display */
+{
+	error(str);		/* Display the passed error message. */
+	longjmp(jmp_save, 13);	/* Abort the current operation with the critical error number 13. */
+}
+
 /*
  * Check if a floating point math function flagged an error.
+ *
+ * There is no return if an error message is displayed.
  */
 void
 check_err()
@@ -95,7 +111,8 @@ init_mem()
 }
 
 /*
- * Initialize the global variables.
+ * Initialize some important global variables to their defaults.
+ * This is called on startup and by process() to reset the global flags to the default state.
  */
 void
 init_gvars()
@@ -107,33 +124,43 @@ init_gvars()
 	sign_cmp_flag = false;
 	approximate_roots = false;
 
+	/* initialize the universal and often used constant "0" expression */
 	zero_token.level = 1;
 	zero_token.kind = CONSTANT;
 	zero_token.token.constant = 0.0;
 
+	/* initialize the universal and often used constant "1" expression */
 	one_token.level = 1;
 	one_token.kind = CONSTANT;
 	one_token.token.constant = 1.0;
 }
 
 /*
- * Clean up when processing is unexpectedly
- * interrupted or terminated.
+ * Clean up when processing is unexpectedly interrupted or terminated.
  */
 void
 clean_up()
 {
-	init_gvars();
-	if (gfp != stdout) {
+	int	i;
+
+	init_gvars();		/* reset the global variables to the default */
+	if (gfp != stdout) {	/* reset the output file to standard output */
 #if	!SECURE
 		fclose(gfp);
 #endif
 		gfp = stdout;
 	}
+	for (i = 0; i < n_equations; i++) {
+		if (n_lhs[i] <= 0) {
+			n_lhs[i] = 0;
+			n_rhs[i] = 0;
+		}
+	}
 }
 
 /*
- * Register all sign variables in all equation spaces.
+ * Register all sign variables in all equation spaces
+ * so that the next sign variables returned by next_sign() will be unique.
  */
 void
 set_sign_array()
@@ -167,7 +194,7 @@ long	*vp;
 {
 	int	i;
 
-	for (i = 2;; i++) {
+	for (i = 0;; i++) {
 		if (i >= ARR_CNT(sign_array)) {
 			/* out of unique sign variables */
 			*vp = SIGN;
@@ -239,14 +266,23 @@ alloc_next_espace()
 {
 	int	i, n;
 
-	for (n = cur_equation, i = 1;; n = (n + 1) % n_equations, i++) {
-		if (i > n_equations) {
-			i = n_equations;
-			if (!alloc_espace(i)) {
+	for (n = cur_equation, i = 0;; n = (n + 1) % N_EQUATIONS, i++) {
+		if (i >= N_EQUATIONS)
+			return -1;
+		if (n >= n_equations) {
+			n = n_equations;
+			if (!alloc_espace(n)) {
+				warning(_("Memory is exhausted."));
+				for (n = 0; n < n_equations; n++) {
+					if (n_lhs[n] == 0) {
+						n_rhs[n] = 0;
+						return n;
+					}
+				}
 				return -1;
 			}
 			n_equations++;
-			return i;
+			return n;
 		}
 		if (n_lhs[n] == 0)
 			break;
@@ -275,12 +311,16 @@ next_espace()
 }
 
 /*
- * Copy equation number "src" to equation number "dest".
+ * Copy equation space "src" to equation space "dest".
  */
 void
 copy_espace(src, dest)
-int	src, dest;
+int	src, dest;	/* equation space numbers */
 {
+	if (src == dest) {
+		error("Internal error: copy_espace() source and destination the same.");
+		return;
+	}
 	blt(lhs[dest], lhs[src], n_lhs[src] * sizeof(token_type));
 	n_lhs[dest] = n_lhs[src];
 	blt(rhs[dest], rhs[src], n_rhs[src] * sizeof(token_type));
@@ -288,7 +328,7 @@ int	src, dest;
 }
 
 /*
- * Return true if equation number "i" is solved for a normal variable.
+ * Return true if equation space "i" is solved for a normal variable.
  */
 int
 solved_equation(i)
@@ -304,7 +344,7 @@ int	i;
 }
 
 /*
- * Return true if variable "v" exists in expression.
+ * Return the number of times variable "v" is found in an expression.
  */
 int
 found_var(p1, n, v)
@@ -313,24 +353,25 @@ int		n;	/* expression length */
 long		v;	/* standard Mathomatic variable */
 {
 	int	j;
+	int	count = 0;
 
 	if (v) {
 		for (j = 0; j < n; j++) {
 			if (p1[j].kind == VARIABLE && p1[j].token.variable == v) {
-				return true;
+				count++;
 			}
 		}
 	}
-	return false;
+	return count;
 }
 
 /*
- * Return true if variable "v" exists in equation number "i".
+ * Return true if variable "v" exists in equation space "i".
  */
 int
 var_in_equation(i, v)
-int	i;
-long	v;
+int	i;	/* equation space number */
+long	v;	/* standard Mathomatic variable */
 {
 	if (n_lhs[i] <= 0)
 		return false;
@@ -424,63 +465,6 @@ char	*cp;
 	return i;
 }
 
-/*
- * Parse an expression with equation space pull if the line starts with "#".
- *
- * Returns the new string position or NULL on error.
- * Currently, there can be no more to parse in the string when this returns.
- */
-char	*
-parse_expr(equation, np, cp)
-token_type	*equation;	/* where the parsed expression is stored (equation side) */
-int		*np;		/* pointer to the returned parsed expression length */
-char		*cp;		/* string to parse */
-{
-	int	i;
-	char	*cp1, *cp2;
-
-	if (!case_sensitive_flag) {
-		str_tolower(cp);
-	}
-	cp1 = skip_space(cp);
-	if (*cp1 == '#') {
-		cp1++;
-		switch (*cp1) {
-		case '+':
-		case '-':
-			i = decstrtol(cp1, &cp2);
-			i = cur_equation + i;
-			break;
-		default:
-			i = decstrtol(cp1, &cp2) - 1;
-			break;
-		}
-		if (cp1 == cp2 || *cp2) {
-			error(_("Error parsing equation space number."));
-			return NULL;
-		}
-		if (i < 0 || i >= n_equations || n_lhs[i] <= 0) {
-			error(_("Equation not defined."));
-			return NULL;
-		}
-		if (n_rhs[i]) {
-			blt(equation, rhs[i], n_rhs[i] * sizeof(token_type));
-			*np = n_rhs[i];
-		} else {
-			blt(equation, lhs[i], n_lhs[i] * sizeof(token_type));
-			*np = n_lhs[i];
-		}
-		list_proc(equation, *np, false);
-		fprintf(gfp, "\n");
-		return cp2;
-	}
-	if ((cp = parse_section(equation, np, cp)) != NULL) {
-		if (extra_characters(cp))
-			return NULL;
-	}
-	return cp;
-}
-
 /*
  * Get an expression from the user.
  * The prompt must be previously copied into the global prompt_str[].
@@ -495,6 +479,12 @@ int		*np;		/* pointer to the returned parsed expression length */
 	char	buf[DEFAULT_N_TOKENS];
 	char	*cp;
 
+#if	LIBRARY
+	snprintf(buf, sizeof(buf), "#%+d", pull_number);
+	pull_number++;
+	cp = parse_expr(equation, np, buf);
+	return(cp && *np > 0);
+#else
 	for (;;) {
 		if ((cp = get_string(buf, sizeof(buf))) == NULL) {
 			return false;
@@ -505,6 +495,7 @@ int		*np;		/* pointer to the returned parsed expression length */
 		}
 	}
 	return(*np > 0);
+#endif
 }
 
 /*
@@ -578,7 +569,7 @@ current_not_defined()
  *
  * Returns "string" if successful or NULL on error.
  */
-char	*
+char *
 get_string(string, n)
 char	*string;	/* storage for input string */
 int	n;		/* maximum size of "string" in bytes */
@@ -597,41 +588,48 @@ int	n;		/* maximum size of "string" in bytes */
 	}
 	input_column = strlen(prompt_str);
 #if	READLINE
-	if (readline_enabled) {
-		rl_inhibit_completion = true;	/* turn off readline file name completion */
+	if (readline_enabled && !echo_input) {
 		cp = readline(prompt_str);
 		if (cp == NULL) {
-			fprintf(gfp, _("\nEnd of input.\n"));
+			if (!quiet_mode)
+				printf(_("\nEnd of input.\n"));
 			exit_program(0);
 		}
 		my_strlcpy(string, cp, n);
-		if (string[0]) {
+		if (skip_space(cp)[0] && (last_history_string == NULL || strcmp(last_history_string, cp))) {
 			add_history(cp);
+			last_history_string = cp;
 		} else {
 			free(cp);
 		}
 	} else {
-		printf("%s", prompt_str);
+		if (!echo_input) {
+			printf("%s", prompt_str);
+		}
 		if (fgets(string, n, stdin) == NULL) {
-			fprintf(gfp, _("\nEnd of input.\n"));
+			if (!quiet_mode)
+				printf(_("\nEnd of input.\n"));
 			exit_program(0);
 		}
 	}
 #else
-	printf("%s", prompt_str);
+	if (!echo_input) {
+		printf("%s", prompt_str);
+	}
 	if (fgets(string, n, stdin) == NULL) {
-		fprintf(gfp, _("\nEnd of input.\n"));
+		if (!quiet_mode)
+			printf(_("\nEnd of input.\n"));
 		exit_program(0);
 	}
 #endif
-	// Fix an fgets() peculiarity:
+	/* Fix an fgets() peculiarity: */
 	i = strlen(string) - 1;
 	if (i >= 0 && string[i] == '\n') {
 		string[i] = '\0';
 	}
-	if (gfp != stdout) {
-		// Input that is prompted for is now included in the redirected output of a command to a file,
-		// making redirection like logging.
+	if (gfp != stdout || echo_input) {
+		/* Input that is prompted for is now included in the redirected output
+		   of a command to a file, making redirection like logging. */
 		fprintf(gfp, "%s%s\n", prompt_str, string);
 	}
 	set_error_level(string);
@@ -666,22 +664,22 @@ get_yes_no()
 }
 
 /*
- * Store or display the result of a command.
+ * Display the result of a command,
+ * or store the pointer to the text of the listed expression
+ * into result_str if compiled for the library.
  *
  * Return true if successful.
  */
 int
 return_result(en)
-int	en;	/* equation space number */
+int	en;	/* equation space number the result is in */
 {
 	if (n_lhs[en] <= 0) {
 		return false;
 	}
 #if	LIBRARY
 	if (gfp == stdout) {
-		if (display2d) {
-			make_fractions_and_group(en);
-		}
+		make_fractions_and_group(en);
 		if (factor_int_flag) {
 			factor_int_sub(en);
 		}
@@ -814,12 +812,14 @@ int	*ip, *jp;
 /*
  * Skip over space characters.
  */
-char	*
+char *
 skip_space(cp)
 char	*cp;	/* character pointer */
 {
-	while (cp && *cp && isspace(*cp))
-		cp++;
+	if (cp) {
+		while (*cp && isspace(*cp))
+			cp++;
+	}
 	return cp;
 }
 
@@ -841,21 +841,31 @@ char	*cp, **cpp;
 }
 
 /*
- * Skip over a parameter (command line argument) in a character string.
- * Parameters are separated with spaces.
- *
- * Returns a string pointer to the next parameter.
+ * Return true if passed character is a Mathomatic command parameter delimiter.
  */
-char	*
+int
+isdelimiter(ch)
+int	ch;
+{
+	return(isspace(ch) || ch == ',' || ch == '=');
+}
+
+/*
+ * Skip over the current parameter in a Mathomatic command line string.
+ * Parameters are usually separated with spaces.
+ *
+ * Returns a string (character pointer) to the next parameter.
+ */
+char *
 skip_param(cp)
-char	*cp;	/* character pointer */
+char	*cp;
 {
 	if (cp) {
-		while (*cp && (!isascii(*cp) || (!isspace(*cp) && *cp != '='))) {
+		while (*cp && (!isascii(*cp) || !isdelimiter(*cp))) {
 			cp++;
 		}
 		cp = skip_space(cp);
-		if (*cp == '=') {
+		if (isdelimiter(*cp)) {
 			cp = skip_space(cp + 1);
 		}
 	}
@@ -863,11 +873,11 @@ char	*cp;	/* character pointer */
 }
 
 /*
- * Compare strings up to the end or the first space.
+ * Compare strings up to the end or the first space or parameter delimiter.
  * Must be an exact match.
  * Compare is alphabetic case insensitive.
  *
- * Returns zero on match, non-zero if different.
+ * Returns zero on exact match, otherwise non-zero if strings are different.
  */
 int
 strcmp_tospace(cp1, cp2)
@@ -875,9 +885,9 @@ char	*cp1, *cp2;
 {
 	char	*cp1a, *cp2a;
 
-	for (cp1a = cp1; *cp1a && !isspace(*cp1a); cp1a++)
+	for (cp1a = cp1; *cp1a && !isdelimiter(*cp1a); cp1a++)
 		;
-	for (cp2a = cp2; *cp2a && !isspace(*cp2a); cp2a++)
+	for (cp2a = cp2; *cp2a && !isdelimiter(*cp2a); cp2a++)
 		;
 	return strncasecmp(cp1, cp2, max(cp1a - cp1, cp2a - cp2));
 }
@@ -889,12 +899,11 @@ int
 level_plus_count(p1, n1, level)
 token_type	*p1;	/* expression pointer */
 int		n1;	/* expression length */
-int		level;	/* parentheses level number */
+int		level;	/* parentheses level number to check */
 {
 	int	i;
-	int	count;
+	int	count = 0;
 
-	count = 0;
 	for (i = 1; i < n1; i += 2) {
 		if (p1[i].level == level) {
 			switch (p1[i].token.operatr) {
@@ -927,9 +936,8 @@ token_type	*p1;	/* expression pointer */
 int		n1;	/* expression length */
 {
 	int	i;
-	int	count;
+	int	count = 0;
 
-	count = 0;
 	for (i = 0; i < n1; i += 2) {
 		if (p1[i].kind == VARIABLE) {
 			count++;
@@ -1029,15 +1037,17 @@ int		n1;	/* expression length */
 }
 
 /*
- * Check for divide by zero.
+ * Check for division by zero.
+ *
+ * Display a warning and return true if passed double is 0.
  */
-void
+int
 check_divide_by_zero(denominator)
 double	denominator;
 {
-#if	!TRAP_FPERRORS
 	if (denominator == 0) {
-		warning(_("Divide by zero."));
+		warning(_("Division by zero."));
+		return true;
 	}
-#endif
+	return false;
 }
