@@ -1,13 +1,9 @@
 /*
- * Main include file for Mathomatic, an algebraic manipulator written entirely in C.
+ * Main include file for Mathomatic.  Can be edited.
  *
- * Copyright (C) 1987-2008 George Gesslein II.
+ * Copyright (C) 1987-2009 George Gesslein II.
  */
 
-#if	LIBRARY
-#define	SILENT	1
-#endif
-
 #ifndef	max
 #define max(a, b)    (((a) > (b)) ? (a) : (b))		/* return the maximum of two values */
 #endif
@@ -25,33 +21,39 @@
 #define	ARR_CNT(a)	((int) (sizeof(a)/sizeof(a[0])))	/* returns the number of elements in an array */
 #define CLEAR_ARRAY(a)	memset(a, 0, sizeof(a))			/* sets all elements of an array to zero */
 
-#define	MAX_K_INTEGER	1.0e14				/* maximum representable integer, 14 digits for doubles */
-#define	MAX_PRECISION	16				/* maximum useful display precision (number of digits) */
+#define	MAX_K_INTEGER	1.0e15				/* maximum safely representable integer, 15 digits for doubles */
 
 #define	blt(dst, src, cnt)	memmove((char *) (dst), (char *) (src), (size_t) (cnt))	/* memory copy function */
 #define always_positive(power)	(fmod((double) (power), 2.0) == 0.0)	/* true if all real numbers raised to "power" result in positive, real numbers */
 
-#if	I18N	/* internationalization */
-#define _(str)		gettext(str)
+#if	I18N		/* internationalization: output in languages other than English using gettext(3) */
+#define _(str)		gettext(str)	/* currently the support code for this incomplete and untested */
 #else
-#define _(str)		str
+#define _(str)		str		/* constant strings to be translated should be marked with this macro */
 #endif
 
 #define	STANDARD_SCREEN_COLUMNS	80			/* default number of columns of characters on the screen */
 #define	STANDARD_SCREEN_ROWS	24			/* default number of lines on the screen */
 
-#define	TMP_FILE	"/tmp/mathomatic.XXXXXX"	/* temp file template */
+#define	TMP_FILE	"/tmp/mathomatic.XXXXXX"	/* temporary file template for mkstemp(3) */
 
-#define	PROMPT		"-> "				/* user interface prompt string */
-#define	HTML_PROMPT	"&mdash;&gt; "
+#define	PROMPT_STR	"-> "				/* user interface main prompt strings, preceded by the current equation number */
+#define	HTML_PROMPT_STR	"&mdash;&gt; "
 
-#define	MAX_CMD_LEN	PATH_MAX			/* maximum command line length (not equations), also max file name length */
+#define	MAX_CMD_LEN	(max(PATH_MAX, 500))		/* maximum command line length (not expression input), also max filename length */
 #define	MAX_PROMPT_LEN	STANDARD_SCREEN_COLUMNS		/* maximum length of prompts */
 
 /*
  * The following defines the maximum number of equation spaces that can be allocated.
+ * This affects maximum memory usage.
  */
+#ifndef	N_EQUATIONS
+#if	HANDHELD
+#define	N_EQUATIONS	50
+#else
 #define	N_EQUATIONS	100
+#endif
+#endif
 
 /*
  * The following defines the default maximum mathematical expression size.
@@ -61,11 +63,18 @@
  * This should be made much smaller for handhelds and embedded systems.
  * Do not set to less than 100.
  */
-#define	DEFAULT_N_TOKENS	30000
+#ifndef	DEFAULT_N_TOKENS
+#if	HANDHELD
+#define	DEFAULT_N_TOKENS	6000
+#else
+#define	DEFAULT_N_TOKENS	60000
+#endif
+#endif
+
 #define	DIVISOR_SIZE		(DEFAULT_N_TOKENS / 2)	/* a nice maximum divisor size */
 
 /*
- * All Mathomatic variables are referenced by the long data type.
+ * All Mathomatic variables are referenced by the value in a C long int variable.
  * The actual name string is stored separately.
  */
 #define	MAX_VAR_NAMES	8000			/* maximum number of variable names, keep this under (VAR_MASK - VAR_OFFSET) */
@@ -81,24 +90,25 @@
 
 typedef	char	sign_array_type[MAX_SUBSCRIPT+2];	/* boolean array for generating unique "sign" variables */
 
-enum kind_list {		/* kinds of tokens specified in the union below */
+enum kind_list {		/* kinds of expression tokens specified in the union below */
 	CONSTANT,
 	VARIABLE,
 	OPERATOR
 };
 
 typedef union {
-	double	constant;	/* internal storage for mathematical constants */
-	long	variable;	/* internal storage for mathematical variables */
-/* predefined special variables follow (order is important) */
+	double	constant;	/* internal storage for Mathomatic numerical constants */
+	long	variable;	/* internal storage for Mathomatic variables */
+/* Predefined special variables follow (order is important): */
 #define	V_NULL		0L	/* null variable, should never be stored in an expression */
-#define	V_E		1L	/* the constant "e" or "e#" */
-#define	V_PI		2L	/* the constant "pi" or "pi#" */
+#define	V_E		1L	/* the symbolic, universal constant "e" or "e#" */
+#define	V_PI		2L	/* the symbolic, universal constant "pi" or "pi#" */
 #define	IMAGINARY	3L	/* the imaginary constant "i" or "i#" */
-#define	SIGN		4L	/* for "sign" variables */
+#define	SIGN		4L	/* for two-valued "sign" variables */
 #define	MATCH_ANY	5L	/* match any variable (wild-card variable) */
-	int	operatr;	/* internal storage for mathematical operators */
-/* valid operators follow (order doesn't matter) */
+#define	V_INTEGER_NAME	"integer"	/* name for the integer variable type */
+	int	operatr;	/* internal storage for Mathomatic operators */
+/* Valid operators follow (order doesn't matter), 0 is reserved for no operator: */
 #define	PLUS		1	/* a + b */
 #define	MINUS		2	/* a - b */
 #define	TIMES		3	/* a * b */
@@ -106,7 +116,7 @@ typedef union {
 #define	MODULUS		5	/* a % b */
 #define	POWER		6	/* a ^ b */
 #define	FACTORIAL	7	/* a! */
-#define	NEGATE		8	/* special parser operator */
+#define	NEGATE		8	/* special parser operator not seen outside parser */
 #define IDIVIDE		9	/* a // b */
 } storage_type;
 
@@ -116,7 +126,7 @@ typedef	struct {		/* storage structure for each token in an expression */
 	storage_type	token;	/* the actual token */
 } token_type;
 
-typedef struct {		/* variable sort data structure */
+typedef struct {		/* qsort(3) data structure for sorting Mathomatic variables */
 	long	v;		/* Mathomatic variable */
 	int	count;		/* number of times the variable occurs */
 } sort_type;
@@ -129,5 +139,5 @@ typedef struct {		/* variable sort data structure */
 #else
 #define list_tdebug(level)		list_debug(level, tlhs, n_tlhs, trhs, n_trhs)
 #define	side_debug(level, p1, n1)	list_debug(level, p1, n1, NULL, 0)
-#define debug_string(level, str)	{ if (debug_level >= (level)) printf("%s\n", str); }
+#define debug_string(level, str)	{ if (debug_level >= (level)) fprintf(gfp, "%s\n", str); }
 #endif
