@@ -1,7 +1,7 @@
 /*
  * Floating point complex number routines specifically for Mathomatic.
  *
- * Copyright (C) 1987-2008 George Gesslein II.
+ * Copyright (C) 1987-2009 George Gesslein II.
  */
 
 #include "includes.h"
@@ -19,7 +19,6 @@ double	x, y, *radiusp, *thetap;
 	*thetap = atan2(y, x);
 }
 
-#if	!LIBRARY
 /*
  * The roots command.
  */
@@ -29,11 +28,12 @@ char	*cp;
 {
 #define	MAX_ROOT	10000.0	/* root limit needed because more roots become more inaccurate and take longer to calculate */
 
-	complexs	c, c2, check;
-	double		d, k;
-	double		root;
-	double		radius, theta;
-	double		radius_root = 0.0;
+	complexs	c, c2;
+#if	!SILENT
+	complexs	check;
+	double		d;
+#endif
+	double		k, root, radius, theta, radius_root = 0.0;
 	char		buf[MAX_CMD_LEN];
 
 	if (*cp == '\0') {
@@ -54,7 +54,7 @@ char	*cp;
 	}
 	c.re = strtod(cp, &cp);
 	if (*cp && !isspace(*cp)) {
-		printf(_("Invalid number.\n"));
+		error(_("Number expected."));
 		return false;
 	}
 	cp = skip_space(cp);
@@ -65,7 +65,7 @@ char	*cp;
 	}
 	c.im = strtod(cp, &cp);
 	if (*cp) {
-		printf(_("Invalid number.\n"));
+		error(_("Number expected."));
 		return false;
 	}
 	if (c.re == 0.0 && c.im == 0.0) {
@@ -78,8 +78,8 @@ char	*cp;
 		radius_root = pow(radius, 1.0 / root);
 	}
 	check_err();
-	fprintf(gfp, _("\nThe polar coordinates before root taking are:\n%.12g amplitude and %.12g radians (%.12g degrees).\n\n"),
-	    radius, theta, theta * 180.0 / M_PI);
+	fprintf(gfp, _("\nThe polar coordinates are:\n%.*g amplitude and %.*g radians (%.*g degrees).\n\n"),
+	    precision, radius, precision, theta, precision, theta * 180.0 / M_PI);
 	if (root) {
 		if (c.im == 0.0) {
 			fprintf(gfp, _("The %.12g roots of (%.12g)^(1/%.12g) are:\n\n"), root, c.re, root);
@@ -91,33 +91,39 @@ char	*cp;
 			c2.re = radius_root * cos((theta + 2.0 * k * M_PI) / root);
 			c2.im = radius_root * sin((theta + 2.0 * k * M_PI) / root);
 			complex_fixup(&c2);
-			if (c2.im == 0.0) {
-				fprintf(gfp, "%.12g\n", c2.re);
-			} else {
-				fprintf(gfp, "%.12g %+.12g*i\n", c2.re, c2.im);
+			if (c2.re || c2.im == 0.0) {
+				fprintf(gfp, "%.12g ", c2.re);
 			}
+			if (c2.im) {
+				fprintf(gfp, "%+.12g*i", c2.im);
+			}
+			fprintf(gfp, "\n");
+#if	!SILENT
 			check = c2;
 			for (d = 1.0; d < root; d += 1.0) {
 				check = complex_mult(check, c2);
 			}
 			complex_fixup(&check);
-			if (check.im == 0.0) {
-				printf(_("Inverse check: %.12g\n\n"), check.re);
-			} else {
-				printf(_("Inverse check: %.12g %+.12g*i\n\n"), check.re, check.im);
+			printf(_("Inverse check:"));
+			if (check.re || check.im == 0.0) {
+				printf(" %.12g", check.re);
 			}
+			if (check.im) {
+				printf(" %+.12g*i", check.im);
+			}
+			printf("\n\n");
+#endif
 		}
 	}
 	return true;
 }
-#endif
 
 /*
- * Approximate roots of complex numbers:
+ * Approximate roots of complex numbers in an equation side:
  * (complex^real) and (real^complex) and (complex^complex).
- * Only gives one root, when there may be many.
+ * This only gives one root, even when there may be many roots.
  *
- * Return true if expression was modified.
+ * Return true and display a warning message if the equation side was modified.
  */
 int
 complex_root_simp(equation, np)
@@ -127,7 +133,7 @@ int		*np;		/* pointer to length of equation side */
 	int		i, j;
 	int		level;
 	int		len;
-	complexs	c, p;
+	complexs	c, p, r;
 	int		modified = false;
 
 start_over:
@@ -148,7 +154,12 @@ start_over:
 		if (c.im == 0.0 && p.im == 0.0)
 			continue;
 		i += len + 1;
-		c = complex_pow(c, p);
+		r = complex_pow(c, p);
+
+#if	false
+		printf("(%.14g+%.14gi)^(%.14g+%.14gi) = %.14g+%.14gi\n", c.re, c.im, p.re, p.im, r.re, r.im);
+#endif
+
 		if (*np + 5 - (i - j) > n_tokens) {
 			error_huge();
 		}
@@ -156,7 +167,7 @@ start_over:
 		*np += 5 - (i - j);
 		equation[j].level = level;
 		equation[j].kind = CONSTANT;
-		equation[j].token.constant = c.re;
+		equation[j].token.constant = r.re;
 		j++;
 		equation[j].level = level;
 		equation[j].kind = OPERATOR;
@@ -165,7 +176,7 @@ start_over:
 		level++;
 		equation[j].level = level;
 		equation[j].kind = CONSTANT;
-		equation[j].token.constant = c.im;
+		equation[j].token.constant = r.im;
 		j++;
 		equation[j].level = level;
 		equation[j].kind = OPERATOR;
@@ -178,18 +189,16 @@ start_over:
 		goto start_over;
 	}
 	if (modified) {
-		warning(_("Complex number root approximated."));
+		warning(_("Complex number roots approximated."));
 	}
 	return modified;
 }
 
 /*
- * Get a constant, if the passed expression is a constant.
- * This code only handles the case of a single constant, where "n" == 1.
- * I don't know how to easily fix this, should at least handle "n" == 3,
- * which would allow 2 constants separated by any operator.
+ * Get a constant, if the passed expression evaluates to a constant.
+ * This should not be called from low level routines.
  *
- * Return true if successful, with number in "*dp".
+ * Return true if successful, with the floating point constant returned in *dp.
  */
 int
 get_constant(p1, n, dp)
@@ -197,24 +206,61 @@ token_type	*p1;	/* expression pointer */
 int		n;	/* length of expression */
 double		*dp;	/* pointer to returned double */
 {
-	if (n != 1)
-		return false;
-	switch (p1[0].kind) {
-	case CONSTANT:
-		*dp = p1[0].token.constant;
-		return true;
-	case VARIABLE:
-		if (var_is_const(p1[0].token.variable, dp)) {
+	int	i, j;
+	int	level;
+	double	d1, d2;
+	int	prev_approx_flag;
+
+	if (n < 1 || (n & 1) != 1) {
+		error_bug("Call to get_constant() has invalid expression length.");
+	}
+	if (n == 1) {
+		switch (p1[0].kind) {
+		case CONSTANT:
+			*dp = p1[0].token.constant;
 			return true;
+		case VARIABLE:
+			if (var_is_const(p1[0].token.variable, dp)) {
+				return true;
+			}
+			break;
+		case OPERATOR:
+			break;
 		}
+	} else if (n >= 3) {
+		level = p1[1].level;
+		if (!get_constant(p1, 1, &d1))
+			return false;
+		for (i = 1; i < n; i = j) {
+			if (p1[i].kind != OPERATOR || p1[i].level > level)
+				return false;
+			level = p1[i].level;
+			for (j = i + 2; j < n && p1[j].level > level; j += 2)
+				;
+			if (!get_constant(&p1[i+1], j - (i + 1), &d2))
+				return false;
+			prev_approx_flag = approximate_roots;
+			approximate_roots = true;
+			if (calc(NULL, &d1, p1[i].token.operatr, d2)) {
+				approximate_roots = prev_approx_flag;
+				if (p1[i].token.operatr == POWER && !domain_check)
+					return false;
+				domain_check = false;
+			} else {
+				approximate_roots = prev_approx_flag;
+				return false;
+			}
+		}
+		*dp = d1;
+		return true;
 	}
 	return false;
 }
 
 /*
- * Parse a complex number expression.
+ * Get the value of a constant complex number expression.
  *
- * If successful return true with complex number in "*cp".
+ * If successful return true with complex number in *cp.
  */
 int
 parse_complex(p1, n, cp)
@@ -225,14 +271,14 @@ complexs	*cp;	/* pointer to returned complex number */
 	int		j;
 	int		imag_cnt = 0, plus_cnt = 0, times_cnt = 0;
 	complexs	c;
-	int		level2;
-	double		junk;
+	int		level, level2;
 
 	if (get_constant(p1, n, &c.re)) {
 		c.im = 0.0;
 		*cp = c;
 		return true;
 	}
+	level = min_level(p1, n);
 	c.re = 0.0;
 	c.im = 1.0;
 	for (j = n - 1; j >= 0; j--) {
@@ -240,7 +286,7 @@ complexs	*cp;	/* pointer to returned complex number */
 		case CONSTANT:
 			break;
 		case VARIABLE:
-			if (var_is_const(p1[j].token.variable, &junk))
+			if (var_is_const(p1[j].token.variable, NULL))
 				break;
 			if (p1[j].token.variable != IMAGINARY)
 				return false;
@@ -282,12 +328,12 @@ complexs	*cp;	/* pointer to returned complex number */
 					c.im = -c.im;
 				}
 			case PLUS:
-				if (++plus_cnt > 1)
+				if (++plus_cnt > 1 || level != level2)
 					return false;
-				if (p1[j-1].level == level2 && get_constant(&p1[j-1], 1, &c.re)) {
-					continue;
+				if (get_constant(p1, j, &c.re)) {
+					goto done;
 				}
-				if (p1[j+1].level == level2 && get_constant(&p1[j+1], 1, &c.re)) {
+				if (get_constant(&p1[j+1], n - (j + 1), &c.re)) {
 					if (p1[j].token.operatr == MINUS)
 						c.re = -c.re;
 					continue;
@@ -297,6 +343,7 @@ complexs	*cp;	/* pointer to returned complex number */
 			return false;
 		}
 	}
+done:
 	if (imag_cnt != 1)
 		return false;
 	*cp = c;
