@@ -1,7 +1,7 @@
 /*
- * Mathomatic differentiation routines and commands.
+ * Mathomatic symbolic differentiation routines and related commands.
  *
- * Copyright (C) 1987-2008 George Gesslein II.
+ * Copyright (C) 1987-2009 George Gesslein II.
  */
 
 #include "includes.h"
@@ -10,10 +10,12 @@ static int	d_recurse();
 
 /*
  * Compute the derivative of an equation side, with respect to variable "v",
- * using the fast, rule based transform method.
- * The result must be simplified by the caller.
+ * using the fast, rule-based transform method.
+ * This is done by recursively applying the proper rule of differentiation
+ * for each operator encountered.
  *
  * Return true if successful.
+ * The result must be simplified by the caller.
  */
 int
 differentiate(equation, np, v)
@@ -41,10 +43,10 @@ long		v;		/* differentiation variable */
  * Symbolically differentiate expression in "equation"
  * (which is a standard equation side) starting at "loc".
  * The current level of parentheses is "level" and
- * do the differentation with respect to variable "v".
+ * do the differentiation with respect to variable "v".
  *
- * Return "true" if successful, "false" if it is beyond this program's
- * capabilities or an error was encountered.
+ * Return true if successful.
+ * Return false if it is beyond this program's capabilities or an error was encountered.
  */
 static int
 d_recurse(equation, np, loc, level, v)
@@ -82,7 +84,7 @@ long		v;
 				break;
 			default:
 /* Oops.  More than one operator on the same level in this expression. */
-				error("Error in d_recurse().");
+				error("Internal error in d_recurse(): differentiating with unparenthesized operators is not allowed.");
 				return false;
 			}
 			op = equation[endloc].token.operatr;
@@ -288,7 +290,7 @@ derivative_cmd(cp)
 char	*cp;
 {
 	int		i;
-	long		v = 0;		/* variable */
+	long		v = 0;		/* Mathomatic variable */
 	long		l1, order = 1;
 	token_type	*source, *dest;
 	int		n1, *nps, *np;
@@ -335,7 +337,7 @@ char	*cp;
 		}
 	}
 	if (no_vars(source, *nps, &v)) {
-		error(_("Current expression contains no variables, result would be zero."));
+		error(_("Current expression contains no variables; the derivative would be zero."));
 		return false;
 	}
 	if (v == 0) {
@@ -344,20 +346,22 @@ char	*cp;
 		}
 	}
 	if (v != MATCH_ANY && !found_var(source, *nps, v)) {
-		error(_("Variable not found, result would be zero."));
+		error(_("Variable not found; the derivative would be zero."));
 		return false;
 	}
 #if	!SILENT
-	list_var(v, 0);
-	if (n_rhs[cur_equation]) {
-		printf(_("Differentiating the RHS with respect to (%s)"), var_str);
-	} else {
-		printf(_("Differentiating with respect to (%s)"), var_str);
-	}
-	if (simplify_flag) {
-		printf(" and simplifying...\n");
-	} else {
-		printf("...\n");
+	if (debug_level >= 0) {
+		list_var(v, 0);
+		if (n_rhs[cur_equation]) {
+			printf(_("Differentiating the RHS with respect to (%s)"), var_str);
+		} else {
+			printf(_("Differentiating with respect to (%s)"), var_str);
+		}
+		if (simplify_flag) {
+			printf(" and simplifying...\n");
+		} else {
+			printf("...\n");
+		}
 	}
 #endif
 	blt(dest, source, *nps * sizeof(token_type));
@@ -370,6 +374,8 @@ char	*cp;
 		}
 		if (simplify_flag) {
 			simpa_side(dest, &n1, true, false);
+		} else {
+			elim_loop(dest, &n1);
 		}
 	}
 	if (n_rhs[cur_equation]) {
@@ -389,7 +395,7 @@ extrema_cmd(cp)
 char	*cp;
 {
 	int		i;
-	long		v = 0;		/* variable */
+	long		v = 0;		/* Mathomatic variable */
 	long		l1, order = 1;
 	token_type	want;
 	token_type	*source;
@@ -426,7 +432,7 @@ char	*cp;
 		}
 	}
 	if (no_vars(source, n, &v)) {
-		error(_("Current expression contains no variables, there are no extrema."));
+		error(_("Current expression contains no variables."));
 		return false;
 	}
 	if (v == 0) {
@@ -435,7 +441,7 @@ char	*cp;
 		}
 	}
 	if (!found_var(source, n, v)) {
-		error(_("Variable not found."));
+		error(_("Variable not found; the derivative would be zero."));
 		return false;
 	}
 	blt(rhs[i], source, n * sizeof(token_type));
@@ -475,7 +481,7 @@ int
 taylor_cmd(cp)
 char	*cp;
 {
-	long		v = 0;			/* variable */
+	long		v = 0;			/* Mathomatic variable */
 	int		i, j, k, i1;
 	int		level;
 	long		l1, n, order = -1L;
@@ -485,7 +491,6 @@ char	*cp;
 	int		our_nlhs, our_nrhs;
 	token_type	*ep, *source, *dest;
 	int		n1, *nps, *np;
-	int		flag = false;
 
 	cp_start = cp;
 	if (current_not_defined()) {
@@ -573,10 +578,15 @@ char	*cp;
 				return false;
 			}
 		} else {
-			printf(_("Derivatives will be taken until they reach zero...\n"));
 			order = LONG_MAX - 1L;
+#if	!SILENT
+			printf(_("Derivatives will be taken until they reach zero...\n"));
+#endif
 		}
 	}
+#if	!SILENT
+	printf(_("Computing...\n"));
+#endif
 	n = 0;
 	i1 = 0;
 	blt(dest, source, *nps * sizeof(token_type));
@@ -641,11 +651,11 @@ loop_again:
 	for (; i1 < n1; i1++)
 		dest[i1].level++;
 	uf_simp(dest, &n1);
-	if (!flag && exp_contains_infinity(dest, n1)) {
-		warning(_("Result invalid because it contains infinity or NaN."));
-		flag = true;
-	}
 	side_debug(1, dest, n1);
+	if (exp_contains_infinity(dest, n1)) {
+		error(_("Result invalid because it contains infinity or NaN."));
+		return false;
+	}
 	if (n < order) {
 		if (n > 0) {
 			if (!differentiate(rhs[our], &our_nrhs, v)) {
@@ -653,7 +663,7 @@ loop_again:
 				return false;
 			}
 		}
-		symb_flag = true;
+		symb_flag = symblify;
 		simpa_side(rhs[our], &our_nrhs, true, true);
 		symb_flag = false;
 		if (our_nrhs != 1 || rhs[our][0].kind != CONSTANT || rhs[our][0].token.constant != 0.0) {
@@ -690,12 +700,10 @@ int
 limit_cmd(cp)
 char	*cp;
 {
-	int		i, j, k;
-	long		v = 0;			/* variable */
+	int		i;
+	long		v = 0;			/* Mathomatic variable */
 	token_type	solved_v, want;
 	char		*cp_start;
-	int		infinity_flag, found_den, complex_den, num, den;	/* flags */
-	int		level;
 
 	cp_start = cp;
 	if (current_not_defined()) {
@@ -748,56 +756,16 @@ char	*cp;
 			return false;
 		}
 	}
-/* copy the current equation to a new equation space and work on the copy: */
+/* copy the current equation to a new equation space, then simplify and work on the copy: */
 	copy_espace(cur_equation, i);
-/* see if the limit expression contains infinity: */
-	simp_loop(tes, &n_tes);
-	infinity_flag = exp_contains_infinity(tes, n_tes);
-/* first fully simplify the current equation and group denominators together: */
+#if	true
 	simpa_side(rhs[i], &n_rhs[i], false, false);
-	group_proc(rhs[i], &n_rhs[i]);
-	found_den = complex_den = false;
-	for (j = 1; j < n_rhs[i]; j += 2) {
-		switch (rhs[i][j].token.operatr) {
-		case DIVIDE:
-		case MODULUS:
-			num = den = false;
-			level = rhs[i][j].level;
-			for (k = j + 2;; k += 2) {
-				if (rhs[i][k-1].kind == VARIABLE && rhs[i][k-1].token.variable == v) {
-/* the limit variable was found in a denominator */
-					den = true;
-					found_den = true;
-				}
-				if (k >= n_rhs[i] || rhs[i][k].level <= level) {
-					break;
-				}
-				switch (rhs[i][k].token.operatr) {
-				case DIVIDE:
-				case MODULUS:
-/* a denominator contains a fraction */
-					complex_den = true;
-					break;
-				}
-			}
-			for (k = j - 1; k >= 0 && rhs[i][k].level >= level; k--) {
-				if (rhs[i][k].kind == VARIABLE && rhs[i][k].token.variable == v) {
-/* the limit variable was found in a numerator */
-					num = true;
-				}
-			}
-			if (num && den) {
-				complex_den = true;
-			}
-			break;
-		}
-	}
-	if (infinity_flag ? (!complex_den && found_den) : (!found_den)) {
-/* in this case, just substitute the limit variable with the limit expression and simplify: */
-		debug_string(0, "Substituting...");
-		subst_var_with_exp(rhs[i], &n_rhs[i], tes, n_tes, v);
-		goto finish_up;
-	}
+#else
+	simple_frac_side(rhs[i], &n_rhs[i]);
+#endif
+/* see if the limit expression is positive infinity: */
+	simp_loop(tes, &n_tes);
+
 	debug_string(0, "Solving...");
 	if (n_tes == 1 && tes[0].kind == CONSTANT && tes[0].token.constant == INFINITY) {
 /* To take the limit to positive infinity, */
@@ -825,8 +793,8 @@ char	*cp;
 /* replace the limit variable (LHS) with the limit expression: */
 	blt(lhs[i], tes, n_tes * sizeof(token_type));
 	n_lhs[i] = n_tes;
-/* symbolically simplify the RHS: */
-	symb_flag = true;
+/* simplify the RHS: */
+	symb_flag = symblify;
 	simpa_side(rhs[i], &n_rhs[i], false, false);
 	symb_flag = false;
 	if (exp_contains_nan(rhs[i], n_rhs[i])) {
@@ -838,11 +806,8 @@ char	*cp;
 		error(_("Can't take the limit because solve failed."));
 		return false;
 	}
-finish_up:
-/* symbolically simplify before returning the result: */
-	symb_flag = true;
+/* simplify before returning the result: */
 	simpa_side(rhs[i], &n_rhs[i], false, false);
-	symb_flag = false;
 	if (exp_contains_nan(rhs[i], n_rhs[i])) {
 		error(_("Unable to take limit; result contains NaN (Not a Number)."));
 		return false;
