@@ -1,7 +1,7 @@
 /*
- * Mathomatic symbolic factorizing routines.
+ * Mathomatic symbolic factorizing routines, not polynomial factoring.
  *
- * Copyright (C) 1987-2008 George Gesslein II.
+ * Copyright (C) 1987-2009 George Gesslein II.
  *
  * There are proper mathematical names for many algebraic rules.
  * I obviously don't know what they are.
@@ -49,14 +49,12 @@ int		*np;
 /*
  * Factor equation side: (a*c + b*c) -> (c*(a + b)).
  *
- * If "v" and "d" equals 0, factor anything,
+ * If "v" and "d" equal 0, factor anything,
  * including identical bases raised to powers (Horner factoring): (x^2 + x) -> (x*(x + 1)).
  * If "d" equals 1.0, only factor identical bases raised to the power of a constant.
  *
- * If "v" is a variable, only factor expressions containing that variable,
- * with no Horner factoring.
- * If "d" is not equal to 0.0 or 1.0, factor only expressions containing "v" raised
- * to the power of "d".
+ * If "v" is a variable, only factor expressions containing that variable, with no Horner factoring.
+ * If "d" is not equal to 0.0 or 1.0, factor only expressions containing "v" raised to the power of "d".
  *
  * Return true if equation side was modified.
  */
@@ -64,7 +62,7 @@ int
 factor_plus(equation, np, v, d)
 token_type	*equation;	/* pointer to beginning of equation side */
 int		*np;		/* pointer to length of equation side */
-long		v;		/* variable */
+long		v;		/* Mathomatic variable */
 double		d;		/* control exponent */
 {
 	return fplus_recurse(equation, np, 0, 1, v, d, false, false);
@@ -779,6 +777,16 @@ int		*np, loc, i1, n1, i2, n2, level;
 	if (se_compare(&equation[i1], i - i1, &equation[i2], j - i2, &diff_sign)) {
 		goto common_base;
 	}
+	if (i < e1 && j < e2) {
+		if (se_compare(&equation[i1], n1, &equation[i2], j - i2, &diff_sign)) {
+			i = e1;
+			goto common_base;
+		}
+		if (se_compare(&equation[i1], i - i1, &equation[i2], n2, &diff_sign)) {
+			j = e2;
+			goto common_base;
+		}
+	}
 	return false;
 
 common_base:
@@ -932,7 +940,7 @@ int		*np, loc, i1, n1, i2, n2, level;
 {
 	int		e1, e2;
 	int		op1, op2;
-	int		pop1;
+	int		pop1 = TIMES;
 	int		start2;
 	int		i, j, k;
 	int		b1, b2;
@@ -963,6 +971,17 @@ int		*np, loc, i1, n1, i2, n2, level;
 		}
 	}
 	start2 = j;
+	if (se_compare(&equation[i+1], e1 - (i + 1), &one_token, 1, &diff_sign)) {
+		return false;
+	}
+	if (se_compare(&equation[i+1], e1 - (i + 1), &equation[j+1], e2 - (j + 1), &diff_sign)) {
+		b1 = i + 1;
+		b2 = j + 1;
+		i = e1;
+		j = e2;
+		goto common_exponent;
+	}
+
 fp_outer:
 	pop1 = equation[i].token.operatr;
 	if (pop1 == POWER)
@@ -970,9 +989,6 @@ fp_outer:
 	b1 = i + 1;
 	if (b1 >= e1)
 		return false;
-#if	false
-	i = e1;
-#else
 	i = b1 + 1;
 	for (; i < e1; i += 2) {
 		if (equation[i].level == (level + 1)
@@ -981,7 +997,6 @@ fp_outer:
 			break;
 		}
 	}
-#endif
 	if (se_compare(&equation[b1], i - b1, &one_token, 1, &diff_sign)) {
 		goto fp_outer;
 	}
@@ -990,9 +1005,6 @@ fp_inner:
 	b2 = j + 1;
 	if (b2 >= e2)
 		goto fp_outer;
-#if	false
-	j = e2;
-#else
 	j = b2 + 1;
 	for (; j < e2; j += 2) {
 		if (equation[j].level == (level + 1)
@@ -1001,7 +1013,6 @@ fp_inner:
 			break;
 		}
 	}
-#endif
 	if (equation[b2-1].token.operatr == POWER) {
 		if (pop1 != TIMES)
 			goto fp_inner;
@@ -1009,6 +1020,7 @@ fp_inner:
 		goto fp_inner;
 	}
 	if (se_compare(&equation[b1], i - b1, &equation[b2], j - b2, &diff_sign)) {
+common_exponent:
 		if (op2 == DIVIDE)
 			diff_sign = !diff_sign;
 		all_divide = (op1 == DIVIDE && diff_sign);
