@@ -3,10 +3,11 @@
  *
  * Everything that depends on the command table goes here.
  *
- * Copyright (C) 1987-2008 George Gesslein II.
+ * Copyright (C) 1987-2009 George Gesslein II.
  */
 
 #include "includes.h"
+#include "license.h"	/* the current license for Mathomatic */
 
 /*
  * The following structure is used for each Mathomatic command.
@@ -18,6 +19,7 @@ typedef	struct {
 					/* function is passed a char pointer and returns true if successful */
 	char	*usage;			/* command syntax text */
 	char	*info;			/* one line description of command */
+	char	*extra;			/* one line extra info on command */
 } com_type;
 
 /*
@@ -27,78 +29,69 @@ static com_type com_list[] = {
 /*	command name,	alternate name,		function,		usage,							information */
 {	"approximate",	NULL,			approximate_cmd,	"[equation-number-ranges]",				"Approximate all numerical values in equation spaces." },
 #if	!LIBRARY
-{	"calculate",	NULL,			calculate_cmd,		"[\"repeat\"] [\"factor\"] [variable iterations]",	"Temporarily plug in values for variables and approximate." },
+{	"calculate",	NULL,			calculate_cmd,		"[\"repeat\"] [\"factor\"] [variable number-of-iterations]",	"Temporarily plug in values for variables and approximate." },
 #endif
-{	"clear",	NULL,			clear_cmd,		"[equation-number-ranges]",				"Delete expressions stored in memory so equation spaces can be reused." },
-#if	!LIBRARY
-{	"code",		NULL,			code_cmd,		"[\"c\" or \"java\" or \"python\" or \"integer\"] [equation-number-ranges]",	"Output C, Java, or Python code for the specified equations." },
-#endif
-{	"compare",	NULL,			compare_cmd,		"equation-number [\"with\" equation-number]",		"Compare two equation spaces to see if they are the same." },
-{	"copy",		NULL,			copy_cmd,		"[equation-number-range]",				"Duplicate the specified equation spaces." },
+{	"clear",	NULL,			clear_cmd,		"[equation-number-ranges]",				"Delete expressions stored in memory so equation spaces can be reused.", "Tip: Use \"clear all\" to quickly restart Mathomatic." },
+{	"code",		NULL,			code_cmd,		"[\"c\" or \"java\" or \"python\" or \"integer\"] [equation-number-ranges]",	"Output C, Java, or Python code for the specified equations.", "Related commands: simplify, optimize, and variables." },
+{	"compare",	NULL,			compare_cmd,		"[\"symbolic\"] equation-number [\"with\" equation-number]",	"Compare two equation spaces to see if they are mathematically the same." },
+{	"copy",		NULL,			copy_cmd,		"[equation-number-range]",				"Duplicate the contents of the specified equation spaces." },
 {	"derivative",	"differentiate",	derivative_cmd,		"[\"nosimplify\"] [variable or \"all\"] [order]",	"Symbolically differentiate and simplify, order times." },
-{	"display",	NULL,			display_cmd,		"[\"factor\"] [equation-number-range]",			"Display equation spaces in multi-line fraction format." },
-#if	!LIBRARY
+{	"display",	NULL,			display_cmd,		"[\"factor\"] [equation-number-ranges]",			"Display equation spaces in pretty multi-line (2D) fraction format." },
 {	"divide",	NULL,			divide_cmd,		"[variable]",						"Prompt for 2 numbers or polynomials and divide. Give result and GCD." },
-#endif
 {	"echo",		NULL,			echo_cmd,		"[text]",						"Output a line of text, followed by a newline." },
 #if	!LIBRARY && (UNIX || CYGWIN) && !SECURE
-{	"edit",		NULL,			edit_cmd,		"[file-name]",						"Edit all equation spaces or an input file." },
+{	"edit",		NULL,			edit_cmd,		"[file-name]",						"Edit all equation spaces or an input file, then read them in." },
 #endif
 {	"eliminate",	NULL,			eliminate_cmd,		"[\"repeat\"] variables or \"all\" [\"using\" equation-number]",	"Substitute the specified variables with solved equations." },
-{	"extrema",	NULL,			extrema_cmd,		"[variable] [order]",					"Find possible local minimums and maximums of the current expression." },
-{	"factor",	NULL,			factor_cmd,		"[\"number\" [integers]] or [equation-number-range] [variables]",	"Factor given integers or factor variables in expressions." },
-{	"fraction",	NULL,			fraction_cmd,		"[equation-number-range]",				"Convert expressions with algebraic fractions into a single fraction." },
-{	"help",		NULL,			help_cmd,		"[topic or command-names]",				"Short, built-in help." },
-{	"imaginary",	NULL,			imaginary_cmd,		"[variable]",						"Copy the imaginary part of an expression (see real command)." },
-{	"integrate",	"integral",		integrate_cmd,		"[\"constant\" or \"definite\"] variable [order]",	"Symbolically integrate polynomials, order times." },
+{	"extrema",	NULL,			extrema_cmd,		"[variable] [order]",					"Show where the slope equals zero." },
+{	"factor",	"collect",		factor_cmd,		"[\"number\" [integers]] or [\"power\"] [equation-number-range] [variables]",	"Factor variables in equation spaces or factor given integers." },
+{	"fraction",	"together",		fraction_cmd,		"[equation-number-range]",				"Convert expression to a single simple fraction, if any fractions are in it." },
+{	"help",		NULL,			help_cmd,		"[topic or command-names]",				"Short, built-in help and reference." },
+{	"imaginary",	NULL,			imaginary_cmd,		"[variable]",						"Copy the imaginary part of the current expression.", "Related command: real." },
+{	"integrate",	"integral",		integrate_cmd,		"[\"constant\" or \"definite\"] variable [order]",	"Symbolically integrate polynomials order times, then simplify." },
 {	"laplace",	NULL,			laplace_cmd,		"[\"inverse\"] variable",				"Compute the Laplace or inverse Laplace transform of polynomials." },
-{	"limit",	NULL,			limit_cmd,		"variable expression",					"Take the limit as variable goes to expression." },
+{	"limit",	NULL,			limit_cmd,		"variable expression",					"Take the limit as variable goes to expression (experimental)." },
 {	"list",		NULL,			list_cmd,		"[\"export\" or \"maxima\"] [equation-number-ranges]",	"Display equation spaces in single-line format." },
-#if	!LIBRARY
-{	"nintegrate",	NULL,			nintegrate_cmd,		"[\"trapezoid\"] variable [partitions]",		"Approximate the definite integral using Simpson's rule."},
-{	"optimize",	NULL,			optimize_cmd,		"[equation-number-range]",				"Split up an equation into smaller, more efficient equations." },
-#endif
+{	"nintegrate",	NULL,			nintegrate_cmd,		"[\"trapezoid\"] variable [partitions]",		"Do numerical definite integration using Simpson's rule."},
+{	"optimize",	NULL,			optimize_cmd,		"[equation-number-range]",				"Split up equations into smaller, more efficient equations.", "Related command: code." },
 #if	!LIBRARY
 {	"pause",	NULL,			pause_cmd,		"[text]",						"Wait for user to press the Enter key. Optionally display a message." },
 #endif
-{	"product",	NULL,			product_cmd,		"variable start end [step-size]",			"Compute the product as variable goes from start to end." },
+{	"product",	NULL,			product_cmd,		"variable start end [step-size]",			"Compute the product as variable goes from start to end.", "Related command: sum." },
 #if	READLINE
-{	"push",		NULL,			push_cmd,		"[equation-number-range]",				"Push specified equation spaces into the readline history." },
+{	"push",		NULL,			push_cmd,		"[equation-number-range]",				"Push the specified expressions into the readline history." },
 #endif
 #if	!LIBRARY
-{	"quit",		"exit",			quit_cmd,		"",							"Terminate this program without saving." },
+{	"quit",		"exit",			quit_cmd,		"[exit-value]",						"Terminate this program without saving." },
 #endif
 #if	!SECURE
 {	"read",		NULL,			read_cmd,		"file-name",						"Read in a text file as if it was typed in." },
 #endif
-{	"real",		NULL,			real_cmd,		"[variable]",						"Copy the real part of an expression (see imaginary command)." },
+{	"real",		NULL,			real_cmd,		"[variable]",						"Copy the real part of the current expression.", "Related command: imaginary." },
 {	"replace",	NULL,			replace_cmd,		"[variables [\"with\" expression]]",			"Substitute variables in the current equation with expressions." },
-#if	!LIBRARY
 {	"roots",	NULL,			roots_cmd,		"root real-part imaginary-part",			"Display all the roots of a complex number." },
-#endif
 #if	!SECURE
-{	"save",		NULL,			save_cmd,		"file-name",						"Save all equation spaces in a text file." },
+{	"save",		NULL,			save_cmd,		"file-name",						"Save all equation spaces in a text file.", "Related command: read." },
 #endif
-{	"set",		NULL,			set_cmd,		"[[\"no\"] option]",					"Set or display various session options." },
+{	"set",		NULL,			set_cmd,		"[[\"no\"] option]",					"Display, set, or save session options." },
 {	"simplify",	NULL,			simplify_cmd,		"[\"sign\"] [\"symbolic\"] [\"quick\"] [\"fraction\"] [equation-number-range]",	"Completely simplify expressions." },
-{	"solve",	NULL,			solve_cmd,		"[\"verify\"] variable or \"0\"",			"Solve the current equation for a variable or for zero." },
-{	"sum",		NULL,			sum_cmd,		"variable start end [step-size]",			"Compute the summation as variable goes from start to end." },
+{	"solve",	NULL,			solve_cmd,		"[\"verify\"] [\"for\"] variable or \"0\"",		"Solve the current equation for a variable or for zero." },
+{	"sum",		NULL,			sum_cmd,		"variable start end [step-size]",			"Compute the summation as variable goes from start to end.", "Related command: product." },
 #if	!LIBRARY
-{	"tally",	NULL,			tally_cmd,		"[\"average\"]",					"Prompt for and add entries, show running (grand) total." },
+{	"tally",	NULL,			tally_cmd,		"[\"average\"]",					"Prompt for and add entries, show total and optionally the average." },
 #endif
 {	"taylor",	NULL,			taylor_cmd,		"variable order point",					"Compute the Taylor series expansion of the current expression." },
-{	"unfactor",	"expand",		unfactor_cmd,		"[\"fully\"] [equation-number-range]",			"Algebraically expand (multiply out) expressions." },
-#if	!LIBRARY
-{	"variables",	NULL,			variables_cmd,		"[\"c\" or \"java\" or \"integer\"] [equation-number-range]",	"Output all variable names within the specified equations." },
-#endif
-{	"version",	NULL,			version_cmd,		"",							"Display version number, compile flags, and memory usage." }
+{	"unfactor",	"expand",		unfactor_cmd,		"[\"fully\"] [\"quick\"] [\"power\"] [equation-number-range]",	"Algebraically expand (multiply out) expressions." },
+{	"variables",	NULL,			variables_cmd,		"[\"c\" or \"java\" or \"integer\"] [equation-number-range]",	"Output all variable names within the specified equations.", "Related command: code." },
+{	"version",	NULL,			version_cmd,		"",							"Display Mathomatic version and license information." }
 };
 
 /*
- * Process equation and expression input in Mathomatic, with no solving
- * and no automatic calculation.
+ * Process mathematical expression input in Mathomatic,
+ * with no solving and no automatic calculation.
  *
- * Parse the equation or expression text in "cp" and place in equation space "n".
+ * Simply parse the equation or expression text in "cp"
+ * and place the result in equation space "n".
  *
  * Return true if successful.
  */
@@ -124,10 +117,13 @@ char	*cp;
 }
 
 /*
- * Process equation and expression input in Mathomatic.
+ * Process equation and expression input in Mathomatic,
+ * either swapping equation sides, selecting an equation space (autoselect),
+ * solving the current equation (autosolve), calculating a numerical expression (autocalc),
+ * or storing.
  *
- * Parse the expression text in "cp" and solve the current equation for it
- * or place it in equation space "n" if it is not a solve variable.
+ * Parse the mathematical expression text in "cp" and perform one of the above operations,
+ * or store it in equation space "n" and display.
  *
  * Return true if successful.
  */
@@ -137,11 +133,19 @@ int	n;
 char	*cp;
 {
 	int	i;
+	char	*cp1, *ep;
+	int	equals_flag = false;
 
-	if (parse_equation(n, cp)) {
+	if ((ep = parse_equation(n, cp))) {
+		for (cp1 = cp; cp1 < ep; cp1++) {
+			if (*cp1 == '=') {
+				equals_flag = true;
+				break;
+			}
+		}
 		if (n_lhs[n] == 0 && n_rhs[n] == 0) {
 			if (strcmp(cp, "=") == 0 && n_lhs[cur_equation] > 0 && n_rhs[cur_equation] > 0) {
-				debug_string(0, _("Swapping equation sides of the current equation..."));
+				debug_string(0, _("Swapping both sides of the current equation..."));
 				n = cur_equation;
 				i = n_lhs[n];
 				blt(tes, lhs[n], n_lhs[n] * sizeof(token_type));
@@ -154,10 +158,11 @@ char	*cp;
 			return true;
 		}
 		if (n_lhs[n] == 0 || n_rhs[n] == 0) {
-			if (autosolve) {
+			if (autosolve || equals_flag) {
 				if ((n_lhs[n] == 1 && ((lhs[n][0].kind == CONSTANT && lhs[n][0].token.constant == 0.0)
-				    || (lhs[n][0].kind == VARIABLE && (lhs[n][0].token.variable & VAR_MASK) > SIGN)))
-				    || n_rhs[n] == 1) {
+				    || (lhs[n][0].kind == VARIABLE && ((lhs[n][0].token.variable & VAR_MASK) > SIGN || equals_flag))))
+				    || (n_rhs[n] == 1 && ((rhs[n][0].kind == CONSTANT && rhs[n][0].token.constant == 0.0)
+				    || rhs[n][0].kind == VARIABLE))) {
 					if (solve_espace(n, cur_equation)) {
 						return return_result(cur_equation);
 					}
@@ -170,12 +175,13 @@ char	*cp;
 				lhs[n][0] = zero_token;
 				goto return_ok;
 			}
-			if (n_lhs[n] == 1 && lhs[n][0].kind == CONSTANT && fmod(lhs[n][0].token.constant, 1.0) == 0.0
+			if (autoselect && n_lhs[n] == 1 && lhs[n][0].kind == CONSTANT && fmod(lhs[n][0].token.constant, 1.0) == 0.0
 			    && lhs[n][0].token.constant > 0.0 && lhs[n][0].token.constant <= n_equations) {
 				/* easy selecting of equation spaces by just typing in the equation number */
+				cur_equation = lhs[n][0].token.constant - 1;
 				n_lhs[n] = 0;
-				n = lhs[n][0].token.constant - 1;
-				goto return_ok;
+				return_result(cur_equation);
+				return true;
 			}
 #if	!LIBRARY
 			if (autocalc) {
@@ -234,10 +240,11 @@ char	*cp;
 	FILE	*fp;
 	int	append_flag;
 
+	init_gvars();		/* make sure we are in the default state */
+	set_sign_array();	/* register all sign variables so that the next ones will be unique */
 	if (cp == NULL) {
 		return false;
 	}
-	set_sign_array();
 	cp_start = cp;
 	cp = skip_space(cp);
 /* handle the equation number selector */
@@ -269,11 +276,12 @@ char	*cp;
 			return parse(i, cp);
 		}
 		cur_equation = i;
-		return return_result(cur_equation);
+		return_result(cur_equation);
+		return true;
 	}
-#if	(UNIX || CYGWIN) && !SECURE
+#if	(UNIX || CYGWIN) && !SECURE && !LIBRARY
 /* handle shell escape */
-	if (*cp == '!') {
+	if (security_level == 0 && *cp == '!') {
 		cp1 = getenv("SHELL");
 		if (cp1 == NULL) {
 			cp1 = "/bin/sh";
@@ -304,61 +312,70 @@ char	*cp;
 		    || (com_list[i].secondary_name && len >= min(4, strlen(com_list[i].secondary_name)) && strncasecmp(cp, com_list[i].secondary_name, len) == 0)) {
 			cp1 = skip_space(cp1);
 			input_column += (cp1 - cp_start);
+/* Copy the command line to buf2 and use it, because the original string may be overwritten. */
 			if (my_strlcpy(buf2, cp1, sizeof(buf2)) >= sizeof(buf2)) {
 				error(_("Command line too long."));
 				return false;
 			}
 			fp = NULL;
 #if	!SECURE
-			/* handle output redirection */
-			append_flag = false;
-			filename = NULL;
-			for (i1 = strlen(buf2) - 1; i1 >= 0; i1--) {
-				if (buf2[i1] == '>') {
-					filename = skip_space(&buf2[i1+1]);
-					if (i1 && buf2[i1-1] == '>') {
-						i1--;
-						append_flag = true;
+			if (security_level < 2) {
+				/* handle output redirection */
+				append_flag = false;
+				filename = NULL;
+				for (i1 = strlen(buf2) - 1; i1 >= 0; i1--) {
+					if (buf2[i1] == '>') {
+						filename = skip_space(&buf2[i1+1]);
+						if (i1 && buf2[i1-1] == '>') {
+							i1--;
+							append_flag = true;
+						}
+						buf2[i1] = '\0';
+						break;
 					}
-					buf2[i1] = '\0';
-					break;
 				}
-			}
-			if (filename) {
-				if (append_flag) {
-					fp = fopen(filename, "a");
-				} else {
-					fp = fopen(filename, "w");
+				if (filename) {
+					if (append_flag) {
+						fp = fopen(filename, "a");
+					} else {
+						fp = fopen(filename, "w");
+					}
+					if (fp == NULL) {
+						error(_("Can't open redirected output file for writing."));
+						return false;
+					}
+					if (gfp != stdout) {
+						fclose(gfp);	/* make sure previous redirection file is closed */
+					}
+					gfp = fp;
 				}
-				if (fp == NULL) {
-					error(_("Can't open redirected output file for writing."));
-					return false;
-				}
-				gfp = fp;
 			}
 #endif
 			remove_trailing_spaces(buf2);
+			pull_number = 1;
 /* execute the command by calling the command function */
 			rv = (*com_list[i].func)(buf2);
 #if	!SECURE
-			if (fp) {	/* if output redirected, close output file */
-				if (gfp != stdout)
-					fclose(gfp);
-				else
-					fclose(fp);
+			if (fp && gfp != stdout) {
+				fclose(gfp);
 				gfp = stdout;
 			}
+#endif
+#if	!SILENT
+			if (!rv && debug_level >= 0) {
+				printf("Command usage: %s %s\n", com_list[i].name, com_list[i].usage);
+			}
 #endif
 			return rv;
 		}
 	}
-/* cp is not a command, so parse the expression or equation */
+/* cp is not a command, so parse the expression */
 	i = next_espace();
 	input_column += (cp - cp_start);
 	return process_parse(i, cp);
 }
 
-#if	(UNIX || CYGWIN) && !SECURE
+#if	(UNIX || CYGWIN) && !SECURE && !LIBRARY
 /*
  * Execute a system command.  Note that system(3) requires "/bin/sh".
  *
@@ -366,10 +383,13 @@ char	*cp;
  */
 int
 shell_out(cp)
-char	*cp;
+char	*cp;	/* system command string */
 {
 	int	rv;
 
+	if (security_level) {
+		error_bug("Shelling out disabled.");
+	}
 	reset_attr();
 	rv = system(cp);
 	printf("\n");
@@ -386,7 +406,7 @@ char	*cp;
  *
  * Return new position in string or NULL if error.
  */
-char	*
+char *
 parse_var2(vp, cp)
 long	*vp;	/* pointer to returned variable in Mathomatic internal format */
 char	*cp;	/* pointer to variable name string */
@@ -418,6 +438,10 @@ int	i;	/* command table index of command */
 		fprintf(gfp, "Alternate name for this command: %s\n", com_list[i].secondary_name);
 		rows++;
 	}
+	if (com_list[i].extra) {
+		fprintf(gfp, "%s\n", com_list[i].extra);
+		rows++;
+	}
 	fprintf(gfp, "\n");
 	return rows;
 }
@@ -448,11 +472,6 @@ next_argument:
 				flag = true;
 			}
 		}
-		if (*cp == '!') {
-			P("! - Shell escape, execute shell and any specified system command.");
-			P("Usage: ![system-command]\n");
-			flag = true;
-		}
 		if (flag) {
 			cp1 = cp = skip_space(cp1);
 			while (*cp1 && !isspace(*cp1))
@@ -462,9 +481,24 @@ next_argument:
 			}
 			return true;
 		}
+		if (strncasecmp(cp, "copyright", cp1 - cp) == 0) {
+			P("Copyright and License Information for Mathomatic");
+			P("------------------------------------------------");
+			fprintf(gfp, "%s", license_string);
+			return true;
+		}
+		if (strncasecmp(cp, "bugs", cp1 - cp) == 0) {
+			P("Report bugs on the Ubuntu Launchpad website:");
+			P("http://launchpad.net/mathomatic\n");
+
+			P("Please include version information from the version command:\n");
+
+			version_report();
+			return true;
+		}
 		if (strncasecmp(cp, "usage", cp1 - cp) == 0) {
-			P("Command Usage Syntax");
-			P("--------------------");
+			P("Mathomatic Command Usage Syntax");
+			P("-------------------------------");
 			for (i = 0, row = 3; i < ARR_CNT(com_list); i++) {
 				fprintf(gfp, "%s %s\n", com_list[i].name, com_list[i].usage);
 				row++;
@@ -476,54 +510,29 @@ next_argument:
 			}
 			return true;
 		}
-		if (strncasecmp(cp, "geometry", cp1 - cp) == 0) {
-			P("Commonly used standard (Euclidean) geometric formulas");
-			P("-----------------------------------------------------");
-			P("Triangle of base \"b\" and height \"h\":");
-			P("    area = b*h/2");
-			P("Rectangle of length \"l\" and width \"w\":");
-			P("    area = l*w                    perimeter = 2*l + 2*w");
-			P("Trapezoid of parallel sides \"a\" and \"b\", and \"d\" distance between them:");
-			P("    area = d*(a + b)/2");
-			P("Circle of radius \"r\":");
-			P("    area = pi*r^2                 perimeter = 2*pi*r");
-			P("    area = perimeter*r/2");
-			P("Rectangular solid of length \"l\", width \"w\", and height \"h\":");
-			P("    volume = l*w*h                surface_area = 2*l*w + 2*l*h + 2*w*h");
-			P("Sphere of radius \"r\":");
-			P("    volume = 4/3*pi*r^3           surface_area = 4*pi*r^2");
-			P("Right circular cylinder of radius \"r\" and height \"h\":");
-			P("    volume = pi*r^2*h             surface_area = 2*pi*r*(h + r)");
-			P("Right circular cone of radius \"r\" and height \"h\":");
-			P("    volume = pi*r^2*h/3");
-			P("    base_surface_area = pi*r^2    side_surface_area = pi*r*(r^2 + h^2)^.5\n");
-
-			P("Convex polygon of \"n\" sides, sum of all interior angles formula:");
-			P("    sum = (n - 2)*180 degrees     sum = (n - 2)*pi radians");
-			return true;
-		}
 		if (strncasecmp(cp, "expressions", cp1 - cp) == 0 || strncasecmp(cp, "equations", cp1 - cp) == 0) {
 			P("To enter an expression or equation, simply type it in at the prompt.");
 			P("Operators have precedence decreasing as indicated:\n");
 
-			P("    - negate");
+			P("    - negate (highest precedence)");
 			P("    ! factorial (gamma function)");
 			P("    ** or ^ power (exponentiation)");
-			P("    * multiply        / divide          % modulus         // integral divide");
-			P("    + add             - subtract");
-			P("    = equate\n");
+			P("    * multiply      / divide        % modulus       // integral divide");
+			P("    + add           - subtract");
+			P("    = equate (lowest precedence)\n");
 
 			P("Variables consist of any combination of letters, digits, and underscores (_).");
 			P("Predefined constants and variables follow:\n");
 
 			P("    e or e# - the universal constant e (2.7182818284...)");
 			P("    pi or pi# - the universal constant pi (3.1415926535...)");
-			P("    i or i# - the imaginary number (square root of -1)");
-			P("The above constants may also be used most anywhere variables are required.");
-			P("    sign, sign1, sign2, sign3, ... - variables that can only be +1 or -1");
+			P("    i or i# - the imaginary unit (square root of -1)");
+			P("The above constants may also be used anywhere variables are required.");
+			P("    sign, sign1, sign2, ... - variables that can only be +1 or -1");
+			P("    integer - variable that can only be an integer value");
 			P("    inf - floating point infinity constant (not a variable)\n");
 
-			P("Absolute value notation \"|x|\" and dual polarity \"+/-\" are understood.");
+			P("Absolute value notation \"|x|\" and dual polarity \"+/-x\" are understood.");
 			return true;
 		}
 		if (is_all(cp)) {
@@ -535,25 +544,23 @@ next_argument:
 					row = 1;
 					if (!pause_cmd(""))
 						return false;
-					printf("\n");
 				}
 			}
-			P("Full documentation can be found at \"http://mathomatic.org/math/doc/\".");
+			fprintf(gfp, "End of list; total of %d different commands.\n", ARR_CNT(com_list));
 			return true;
 		}
 		error(_("Unrecognized help topic or command."));
 		return false;
 	}
 	/* default help text: */
-	P("Mathomatic is a small Computer Algebra System (CAS).");
-	P("This help command is provided as a quick reference.");
+	P("Mathomatic is a Computer Algebra System (CAS) and calculator program.");
+	P("Type \"help bugs\" for info on how to report bugs found in this program.");
 	P("Type \"help equations\" for help with entering expressions and equations.");
 	P("Type \"help all\" for a summary and syntax of all commands.");
 	P("Type \"help usage\" to display the syntax of all commands.");
-	P("Type \"help geometry\" for some commonly used geometric formulas.");
 	P("\"help\" or \"?\" followed by command names will give info on those commands.\n");
 
-	P("Available commands:");
+	fprintf(gfp, "These are the %d commands:\n", ARR_CNT(com_list));
 	for (i = 0; i < ARR_CNT(com_list); i++) {
 		if ((i % 5) == 0)
 			fprintf(gfp, "\n");
