@@ -2,32 +2,34 @@
  * Mathomatic expression and equation display routines.
  * Color mode routines, too.
  *
- * Copyright (C) 1987-2008 George Gesslein II.
+ * Copyright (C) 1987-2009 George Gesslein II.
  */
 
 #include "includes.h"
 
 #define	EQUATE_STRING	" = "	/* string displayed between the LHS and RHS of equations */
-#define MODULUS_STRING	" % "	/* string displayed for the modulus binary operator */
+#define MODULUS_STRING	" % "	/* string displayed for the modulus operator */
 
 #define	APPEND(str)	{ if (string) { strcpy(&string[len], str); } len += strlen(str); }
 
 static int	flist();
 static int	flist_recurse();
 
-/* ANSI color code array for 8 color ANSI, we don't use black */
+/* ANSI terminal color code array for 8 color ANSI, we don't use black or white */
+/* because the background may be the same color. */
 static int	carray[] = {
 	32,	/* green (default color) */
 	33,	/* yellow */
 	31,	/* red */
 	35,	/* magenta */
 	34,	/* blue */
-	36,	/* cyan */
-	37	/* white */
+	36	/* cyan */
 };
 
-/* HTML color array */
-static char	*html_carray[] = {
+/* Bright HTML color array. */
+/* Used when HTML output and "set color" and "set bold" options are enabled. */
+/* Good looking with a dark background. */
+static char	*bright_html_carray[] = {
 	"#00FF00",	/* bright green (default color) */
 	"#FFFF00",
 	"#FF0000",
@@ -37,12 +39,26 @@ static char	*html_carray[] = {
 	"#0000FF"
 };
 
-/* global variables for flist() */
-int	cur_line;	/* current line */
-int	cur_pos;	/* current position in the current line on the screen */
+/* Dim HTML color array for color HTML output. */
+/* Good looking with a white background. */
+static char	*html_carray[] = {
+	"green",	/* green (default color) */
+	"olive",
+	"red",
+	"maroon",
+	"purple",
+	"teal",
+	"navy"
+};
+
+/* global variables for flist() below */
+static int	cur_line;	/* current line */
+static int	cur_pos;	/* current position in the current line on the screen */
 
 /*
+ * Reset terminal attributes function.
  * Turn color off if color mode is on.
+ * Subsequent output will have no color.
  */
 void
 reset_attr()
@@ -55,11 +71,14 @@ reset_attr()
 			printf("\033[0m");
 		}
 	}
-	fflush(NULL);	/* flush all output */
+	if (fflush(NULL)) {	/* flush all output */
+		perror("fflush(3) call failed");
+	}
 }
 
 /*
  * Set the current color on the display.
+ * Subsequent output will have the color in carray[color].
  * Range is 0 to INT_MAX.
  */
 void
@@ -75,7 +94,11 @@ int	color;
 			if (cur_color >= 0) {
 				printf("</font>");
 			}
-			printf("<font color=\"%s\">", html_carray[color%ARR_CNT(html_carray)]);
+			if (bold_colors) {
+				printf("<font color=\"%s\">", bright_html_carray[color%ARR_CNT(bright_html_carray)]);
+			} else {
+				printf("<font color=\"%s\">", html_carray[color%ARR_CNT(html_carray)]);
+			}
 		} else {
 			printf("\033[%d;%dm", bold_colors, carray[color%ARR_CNT(carray)]);
 		}
@@ -84,7 +107,8 @@ int	color;
 }
 
 /*
- * Set the normal text color.
+ * Set normal text color for subsequent output.
+ * Currently, the normal color (0) is green.
  */
 void
 default_color()
@@ -119,15 +143,11 @@ int	export_flag;	/* non-zero for exportable format (readable by other math progr
 	if (export_flag == 1) {
 		len += fprintf(gfp, ";");
 	}
-	if (high_prec || export_flag) {
 #if	CYGWIN
-		fprintf(gfp, "\r\n");	/* might be redirecting to a Microsoft text file */
+	fprintf(gfp, "\r\n");	/* might be redirecting to a Microsoft text file */
 #else
-		fprintf(gfp, "\n");
+	fprintf(gfp, "\n");
 #endif
-	} else {
-		fprintf(gfp, "\n\n");
-	}
 	return len;
 }
 
@@ -143,18 +163,15 @@ int	n;	/* equation space number */
 {
 	if (n_lhs[n] <= 0)
 		return 0;
+	make_fractions_and_group(n);
+	if (factor_int_flag) {
+		factor_int_sub(n);
+	}
 	if (display2d) {
 		/* display in fraction format */
-		make_fractions_and_group(n);
-		if (factor_int_flag) {
-			factor_int_sub(n);
-		}
 		return flist_equation(n);
 	} else {
 		/* display in single-line format */
-		if (factor_int_flag) {
-			factor_int_sub(n);
-		}
 		return list1_sub(n, false);
 	}
 }
@@ -169,13 +186,13 @@ token_type	*p2;
 int		n2;
 {
 	if (debug_level >= level) {
-		printf(_("level %d: "), level);
+		fprintf(gfp, _("level %d: "), level);
 		list_proc(p1, n1, false);
 		if (p2 && n2 > 0) {
-			printf(EQUATE_STRING);
+			fprintf(gfp, EQUATE_STRING);
 			list_proc(p2, n2, false);
 		}
-		printf("\n");
+		fprintf(gfp, "\n");
 	}
 }
 #endif
@@ -198,13 +215,12 @@ long	v;		/* variable to convert */
 int	lang_code;	/* language code */
 {
 	int		j;
-	char		*cp, buf[100];
+	char		*cp = NULL, buf[100];
 
-	cp = NULL;
+	var_str[0] = '\0';
 	switch (v & VAR_MASK) {
 	case V_NULL:
-		cp = "null";
-		break;
+		return(0);
 	case SIGN:
 		cp = "sign";
 		break;
@@ -283,20 +299,20 @@ int	lang_code;	/* language code */
 	}
 	if (cp) {
 		my_strlcpy(var_str, cp, sizeof(var_str));
+		j = (v >> VAR_SHIFT) & SUBSCRIPT_MASK;
+		if (j) {
+			snprintf(buf, sizeof(buf), "%d", j - 1);
+			strcat(var_str, buf);
+		}
 	} else {
 		my_strlcpy(var_str, "bad_variable", sizeof(var_str));
 	}
-	j = (v >> VAR_SHIFT) & SUBSCRIPT_MASK;
-	if (j) {
-		snprintf(buf, sizeof(buf), "%d", j - 1);
-		strcat(var_str, buf);
-	}
 	return(strlen(var_str));
 }
 
 /*
  * Display an expression in single-line format.
- * Use color if available.
+ * Use color if color mode is set.
  * The expression is not modified.
  *
  * Return number of characters output (excluding escape sequences).
@@ -341,9 +357,17 @@ int		export_flag;	/* flag for exportable format (usually false) */
 					len += fprintf(gfp, "%.20g", p1[i].token.constant);
 				}
 			} else if (finance_option) {
-				len += fprintf(gfp, "%.*f", finance_option, p1[i].token.constant);
+				if (p1[i].token.constant < 0.0) {
+					len += fprintf(gfp, "(%.*f)", finance_option, p1[i].token.constant);
+				} else {
+					len += fprintf(gfp, "%.*f", finance_option, p1[i].token.constant);
+				}
 			} else {
-				len += fprintf(gfp, "%.*g", precision, p1[i].token.constant);
+				if (p1[i].token.constant < 0.0 && !negate_highest_precedence) {
+					len += fprintf(gfp, "(%.*g)", precision, p1[i].token.constant);
+				} else {
+					len += fprintf(gfp, "%.*g", precision, p1[i].token.constant);
+				}
 			}
 			break;
 		case VARIABLE:
@@ -511,9 +535,17 @@ int		export_flag;
 					snprintf(buf, sizeof(buf), "%.20g", p1[i].token.constant);
 				}
 			} else if (finance_option) {
-				snprintf(buf, sizeof(buf), "%.*f", finance_option, p1[i].token.constant);
+				if (p1[i].token.constant < 0.0) {
+					snprintf(buf, sizeof(buf), "(%.*f)", finance_option, p1[i].token.constant);
+				} else {
+					snprintf(buf, sizeof(buf), "%.*f", finance_option, p1[i].token.constant);
+				}
 			} else {
-				snprintf(buf, sizeof(buf), "%.*g", precision, p1[i].token.constant);
+				if (p1[i].token.constant < 0.0 && !negate_highest_precedence) {
+					snprintf(buf, sizeof(buf), "(%.*g)", precision, p1[i].token.constant);
+				} else {
+					snprintf(buf, sizeof(buf), "%.*g", precision, p1[i].token.constant);
+				}
 			}
 			APPEND(buf);
 			break;
@@ -658,16 +690,14 @@ int		int_flag;	/* integer arithmetic flag, should work with any language */
 								if (int_flag || language > 1)
 									break;
 								if (language) {
-									cp = "Math.pow";
+									len += fprintf(gfp, "Math.pow");
 								} else {
-									cp = "pow";
+									len += fprintf(gfp, "pow");
 								}
-								len += fprintf(gfp, cp);
 							}
 							break;
 						case FACTORIAL:
-							cp = "fact";
-							len += fprintf(gfp, cp);
+							len += fprintf(gfp, "fact");
 							break;
 						}
 						break;
@@ -681,10 +711,11 @@ int		int_flag;	/* integer arithmetic flag, should work with any language */
 			if (equation[i].token.constant == 0.0) {
 				equation[i].token.constant = 0.0; /* fix -0 */
 			}
+/* note: here we will need to parenthesize negative numbers to make -2**x work the same with Python: */
 			if (int_flag) {
 				snprintf(buf, sizeof(buf), "%.0f", equation[i].token.constant);
 			} else {
-				snprintf(buf, sizeof(buf), "%#.*g", MAX_PRECISION, equation[i].token.constant);
+				snprintf(buf, sizeof(buf), "%#.*g", DBL_DIG, equation[i].token.constant);
 				j = strlen(buf) - 1;
 				for (; j >= 0; j--) {
 					if (buf[j] == '0')
@@ -696,7 +727,11 @@ int		int_flag;	/* integer arithmetic flag, should work with any language */
 					}
 				}
 			}
-			len += fprintf(gfp, "%s", buf);
+			if (equation[i].token.constant < 0) {
+				len += fprintf(gfp, "(%s)", buf);
+			} else {
+				len += fprintf(gfp, "%s", buf);
+			}
 			break;
 		case VARIABLE:
 			len += list_var(equation[i].token.variable, language + 1);
@@ -744,8 +779,8 @@ int		int_flag;	/* integer arithmetic flag, should work with any language */
 }
 
 /*
- * Display an equation space in multi-line fraction format.
- * Use color if available.
+ * Display an equation space in 2D multi-line fraction format.
+ * Use color if color mode is set.
  *
  * Return the total width of the output (that is, the required number of screen columns)
  * or zero on failure.
@@ -798,7 +833,13 @@ make_smaller:
 	width = max(len + len2, len3);
 	if (screen_columns && gfp == stdout && width >= screen_columns) {
 		/* output too wide to fit screen, output in single-line format */
-		return list1_sub(n, false);
+		width = list1_sub(n, false);
+#if	CYGWIN
+		fprintf(gfp, "\r\n");	/* Be consistent with list1_sub() output. */
+#else
+		fprintf(gfp, "\n");
+#endif
+		return width;
 	}
 	fprintf(gfp, "\n");
 	for (cur_line = max_line; cur_line >= min_line; cur_line--) {
@@ -818,6 +859,7 @@ make_smaller:
 		fprintf(gfp, "\n");
 	}
 	if (sind < n_rhs[n]) {
+/* output second half of split equation that was too wide to display on the screen without splitting */
 		fprintf(gfp, "\n");
 		for (cur_line = max2_line; cur_line >= min2_line; cur_line--) {
 			cur_pos = 0;
@@ -830,7 +872,9 @@ make_smaller:
 }
 
 /*
- * Display a line of an expression if "out_flag" is true.
+ * Display a line of an expression in 2D fraction format if "out_flag" is true.
+ * The number of the line to output is stored in the global variable cur_line.
+ * 0 is the middle line, lines above are positive, lines below are negative.
  * Use color if available.
  *
  * Return the width of the expression (that is, the required number of screen columns).
@@ -870,7 +914,7 @@ int		*highp, *lowp;
 	int	len_div;
 	int	level;
 	int	start_level;
-	int	oflag;
+	int	oflag, html_out;
 	int	len = 0, len1, len2;
 	int	high, low;
 	char	buf[500];
@@ -883,6 +927,7 @@ int		*highp, *lowp;
 		return 0;
 	}
 	oflag = (out_flag && line == cur_line);
+	html_out = (html_flag && gfp == stdout);
 	if (oflag) {
 		set_color(cur_level-1);
 		for (; cur_pos < pos; cur_pos++) {
@@ -964,7 +1009,7 @@ check_again:
 			if (oflag) {
 				set_color(level-1);
 				for (j = 0; j < len_div; j++) {
-					if (html_flag) {
+					if (html_out) {
 						fprintf(gfp, "&mdash;");
 					} else {
 						fprintf(gfp, "-");
@@ -990,9 +1035,17 @@ check_again:
 				p1[i].token.constant = 0.0; /* fix -0 */
 			}
 			if (finance_option) {
-				len += snprintf(buf, sizeof(buf), "%.*f", finance_option, p1[i].token.constant);
+				if (p1[i].token.constant < 0.0) {
+					len += snprintf(buf, sizeof(buf), "(%.*f)", finance_option, p1[i].token.constant);
+				} else {
+					len += snprintf(buf, sizeof(buf), "%.*f", finance_option, p1[i].token.constant);
+				}
 			} else {
-				len += snprintf(buf, sizeof(buf), "%.*g", precision, p1[i].token.constant);
+				if (p1[i].token.constant < 0.0 && !negate_highest_precedence) {
+					len += snprintf(buf, sizeof(buf), "(%.*g)", precision, p1[i].token.constant);
+				} else {
+					len += snprintf(buf, sizeof(buf), "%.*g", precision, p1[i].token.constant);
+				}
 			}
 			if (oflag)
 				fprintf(gfp, "%s", buf);
@@ -1003,14 +1056,14 @@ check_again:
 				fprintf(gfp, "%s", var_str);
 			break;
 		case OPERATOR:
-			/* len must be incremented here by the number of columns the display of the operator takes */
+/* "len" must be incremented here by the number of columns the display of the operator takes */
 			switch (p1[i].token.operatr) {
 			case PLUS:
 				cp = " + ";
 				len += 3;
 				break;
 			case MINUS:
-				if (html_flag) {
+				if (html_out) {
 					cp = " &minus; ";
 				} else {
 					cp = " - ";
@@ -1018,7 +1071,7 @@ check_again:
 				len += 3;
 				break;
 			case TIMES:
-				if (html_flag) {
+				if (html_out) {
 					cp = "&middot;";
 				} else {
 					cp = "*";
@@ -1038,6 +1091,25 @@ check_again:
 				len += strlen(cp);
 				break;
 			case POWER:
+#if	false
+				if (html_out
+				    && p1[i-1].level == p1[i].level
+				    && p1[i+1].level == p1[i].level
+				    && p1[i+1].kind == CONSTANT) {
+					if (p1[i+1].token.constant == 2.0) {
+						cp = "&sup2;";
+						len++;
+						i++;
+						break;
+					}
+					if (p1[i+1].token.constant == 3.0) {
+						cp = "&sup3;";
+						len++;
+						i++;
+						break;
+					}
+				}
+#endif
 				if (power_starstar) {
 					cp = "**";
 					len += 2;
