@@ -1,7 +1,7 @@
 /*
  * Expression parsing routines for Mathomatic.
  *
- * Copyright (C) 1987-2008 George Gesslein II.
+ * Copyright (C) 1987-2009 George Gesslein II.
  */
 
 #include "includes.h"
@@ -17,9 +17,11 @@ void
 str_tolower(cp)
 char	*cp;
 {
-	for (; cp && *cp; cp++) {
-		if (isupper(*cp))
-			*cp = tolower(*cp);
+	if (cp) {
+		for (; *cp; cp++) {
+			if (isupper(*cp))
+				*cp = tolower(*cp);
+		}
 	}
 }
 
@@ -34,7 +36,7 @@ char	*cp;	/* error message */
 #if	!SILENT
 	int	 i;
 
-	if (isatty(0) && point_flag) {
+	if (isatty(0) && point_flag && !html_flag) {
 		for (i = 0; i < (input_column + cnt); i++) {
 			printf(" ");
 		}
@@ -51,7 +53,7 @@ int
 isvarchar(ch)
 int	ch;
 {
-	return(ch == '_' || (special_variable_character && ch == special_variable_character) || isalpha(ch));
+	return(ch == '_' || (ch && strchr(special_variable_characters, ch)) || isalpha(ch));
 }
 
 /*
@@ -66,10 +68,9 @@ int		i;	/* location of operator to parenthesize in expression */
 	int	j;
 	int	level;
 
-#if	false
+#if	true
 	if (i >= (n - 1) || (n & 1) != 1 || (i & 1) != 1) {
-		error("Internal error in arguments to binary_parenthesize().");
-		longjmp(jmp_save, 13);
+		error_bug("Internal error in arguments to binary_parenthesize().");
 	}
 #endif
 	level = p1[i].level++;
@@ -104,9 +105,10 @@ int		*np;
 		if (equation[i].token.operatr == NEGATE) {
 			/* change negate operator to -1 times the operand: */
 			equation[i].token.operatr = TIMES;
-			/* make negate top priority by parenthesizing it first: */
-			binary_parenthesize(equation, *np, i);
-			/* comment out the above line to make negate the same priority as times/divide */
+			if (negate_highest_precedence) {
+				/* make negate top priority by parenthesizing it first: */
+				binary_parenthesize(equation, *np, i);
+			}	/* otherwise negate has same priority as times/divide */
 		}
 	}
 }
@@ -114,6 +116,8 @@ int		*np;
 /*
  * Parenthesize most operators so that expression evaluation is in the correct order.
  * Gives different operators on the same level in an expression the correct priority.
+ * Similar operators on the same level are always evaluated or grouped left to right,
+ * except for the power operator.
  *
  * organize() should be called after this to remove unneeded parentheses.
  */
@@ -132,13 +136,13 @@ int		*np;		/* pointer to expression length */
 	}
 
 	if (right_associative_power) {
-		for (i = *np - 2; i >= 1; i -= 2) {	/* count down (right to left) */
+		for (i = *np - 2; i >= 1; i -= 2) {	/* count down (for right to left evaluation) */
 			if (equation[i].token.operatr == POWER) {
 				binary_parenthesize(equation, *np, i);
 			}
 		}
 	} else {
-		for (i = 1; i < *np; i += 2) {		/* count up (left to right) */
+		for (i = 1; i < *np; i += 2) {		/* count up (for left to right evaluation) */
 			if (equation[i].token.operatr == POWER) {
 				binary_parenthesize(equation, *np, i);
 			}
@@ -157,48 +161,25 @@ int		*np;		/* pointer to expression length */
 	}
 }
 
-/*
- * Parse an equation string into equation space "n".
- *
- * Returns the new string position or NULL on error.
- * Currently, there can be no more to parse in the string when this returns.
- */
-char	*
-parse_equation(n, cp)
-int	n;	/* equation space number */
-char	*cp;	/* pointer to the beginning of the equation character string */
-{
-	if (!case_sensitive_flag) {
-		str_tolower(cp);
-	}
-	if ((cp = parse_section(lhs[n], &n_lhs[n], cp)) != NULL) {
-		if ((cp = parse_section(rhs[n], &n_rhs[n], cp)) != NULL) {
-			if (extra_characters(cp))
-				return NULL;
-			return cp;
-		}
-	}
-	return NULL;
-}
-
 /*
  * This is a simple, non-recursive mathematical expression parser.
  * To parse, the character string is sequentially read and stored
  * in the internal storage format.
- * The maximum length that can be parsed is the size of an equation side.
+ * The maximum length that can be parsed is the size of an equation side (n_tokens).
  * Any syntax and other errors are carefully reported and give a NULL return.
  *
- * Returns the new string position or NULL if error.
+ * Returns the new string position, or NULL if error.
  */
-char	*
-parse_section(equation, np, cp)
+char *
+parse_section(equation, np, cp, allow_space)
 token_type	*equation;	/* where the parsed expression is stored (equation side) */
 int		*np;		/* pointer to the returned parsed expression length */
 char		*cp;		/* string to parse */
+int		allow_space;	/* if false, any space characters terminate parsing */
 {
-	int		n = 0;		/* position in equation[] */
-	int		cur_level = 1;	/* current level of parentheses */
-	int		operand;	/* flip-flop between operand and operator */
+	int		n = 0;			/* position in equation[] */
+	int		cur_level = 1;		/* current level of parentheses */
+	int		operand = false;	/* flip-flop between operand and operator */
 	char		*cp_start, *cp1;
 	double		d;
 	int		abs_count = 0;
@@ -207,41 +188,47 @@ char		*cp;		/* string to parse */
 	if (cp == NULL)
 		return(NULL);
 	cp_start = cp;
-	operand = false;
 	for (;; cp++) {
+		if (n > (n_tokens - 10)) {
+			error_huge();
+		}
 		switch (*cp) {
-		case ' ':
-		case '\t':
-			continue;
 		case '(':
-		case '[':
 		case '{':
-			cur_level++;
 			if (operand) {
+#if	true
+				operand = false;
+				equation[n].level = cur_level;
+				equation[n].kind = OPERATOR;
+				equation[n].token.operatr = TIMES;
+				n++;
+#else
 				goto syntax_error;
+#endif
 			}
+			cur_level++;
 			continue;
 		case ')':
-		case ']':
 		case '}':
 			cur_level--;
 			if (cur_level <= 0 || (abs_count > 0 && cur_level < abs_array[abs_count-1])) {
-				put_up_arrow((int) (cp - cp_start), _("Unmatched parentheses: too many )."));
+				put_up_arrow(cp - cp_start, _("Unmatched parenthesis: too many )"));
 				return(NULL);
 			}
 			if (!operand) {
 				goto syntax_error;
 			}
 			continue;
+		case ' ':
+		case '\t':
+			if (allow_space)
+				continue;
 		case '=':
 		case ';':
 		case '\0':
 		case '\n':
 			goto p_out;	/* terminator encountered */
 		}
-		if (n > (n_tokens - 6)) {
-			error_huge();
-		}
 		operand = !operand;
 		switch (*cp) {
 		case '|':
@@ -283,7 +270,7 @@ char		*cp;		/* string to parse */
 				goto syntax_error;
 			}
 			if (cp[1] == '!' && cp[2] != '!') {
-				warning(_("Multifactorial not implemented, use \"(x!)!\" for the factorial of x factorial."));
+				warning(_("Multifactorial not implemented, using x!! = (x!)!"));
 			}
 			equation[n].level = cur_level;
 			equation[n].kind = OPERATOR;
@@ -385,7 +372,7 @@ parse_power:
 				equation[n].token.operatr = TIMES;
 				n++;
 			}
-			if (*cp == '-' && !(isdigit(cp[1]) || cp[1] == '.')) {
+			if (*cp == '-' && (!negate_highest_precedence || !(isdigit(cp[1]) || cp[1] == '.'))) {
 				equation[n].kind = CONSTANT;
 				equation[n].token.constant = -1.0;
 				equation[n].level = cur_level;
@@ -401,11 +388,10 @@ parse_power:
 			errno = 0;
 			d = strtod(cp1, &cp);
 			if (cp == cp1) {
-				put_up_arrow((int) (cp1 - cp_start), _("Error parsing constant."));
-				return(NULL);
+				goto syntax_error;
 			}
 			if (errno) {
-				put_up_arrow((int) (cp1 - cp_start), _("Constant out of range."));
+				put_up_arrow(cp1 - cp_start, _("Constant out of range."));
 				return(NULL);
 			}
 			equation[n].kind = CONSTANT;
@@ -416,7 +402,7 @@ parse_power:
 			break;
 		default:
 			if (!isvarchar(*cp)) {
-				put_up_arrow((int) (cp - cp_start), _("Unrecognized character."));
+				put_up_arrow(cp - cp_start, _("Unrecognized character."));
 				return(NULL);
 			}
 			if (!operand) {
@@ -426,6 +412,7 @@ parse_power:
 				equation[n].token.operatr = TIMES;
 				n++;
 			}
+			cp1 = cp;
 			if (strncasecmp(cp, INFINITY_NAME, strlen(INFINITY_NAME)) == 0
 			    && !isvarchar(cp[strlen(INFINITY_NAME)])) {
 				equation[n].kind = CONSTANT;
@@ -433,12 +420,15 @@ parse_power:
 				cp += strlen(INFINITY_NAME);
 			} else {
 				equation[n].kind = VARIABLE;
-				cp1 = cp;
 				cp = parse_var(&equation[n].token.variable, cp);
 				if (cp == NULL) {
 					return(NULL);
 				}
 			}
+			if (*cp == '(') {
+				put_up_arrow(cp1 - cp_start, _("Named functions currently not implemented."));
+				return(NULL);
+			}
 			cp--;
 			equation[n].level = cur_level;
 			n++;
@@ -450,7 +440,7 @@ p_out:
 		goto syntax_error;
 	}
 	if (cur_level != 1) {
-		put_up_arrow((int) (cp - cp_start), _("Unmatched parentheses: missing )."));
+		put_up_arrow(cp - cp_start, _("Unmatched parenthesis: missing )"));
 		return(NULL);
 	}
 	while (*cp == '=')
@@ -465,10 +455,93 @@ p_out:
 	return cp;
 
 syntax_error:
-	put_up_arrow((int) (cp - cp_start), _("Syntax error."));
+	put_up_arrow(cp - cp_start, _("Syntax error."));
 	return(NULL);
 }
 
+/*
+ * Parse an equation string into equation space "n".
+ *
+ * Returns the new string position, or NULL if error.
+ * Currently, there can be no more to parse in the string when this returns.
+ */
+char *
+parse_equation(n, cp)
+int	n;	/* equation space number */
+char	*cp;	/* pointer to the beginning of the equation character string */
+{
+	if (!case_sensitive_flag) {
+		str_tolower(cp);
+	}
+	if ((cp = parse_section(lhs[n], &n_lhs[n], cp, true)) != NULL) {
+		if ((cp = parse_section(rhs[n], &n_rhs[n], cp, true)) != NULL) {
+			if (extra_characters(cp))
+				return NULL;
+			return cp;
+		}
+	}
+	return NULL;
+}
+
+/*
+ * Parse an expression with equation space pull if the line starts with "#" followed by an equation number.
+ *
+ * Returns the new string position, or NULL if error.
+ * Currently, there can be no more to parse in the string when this returns.
+ */
+char *
+parse_expr(equation, np, cp)
+token_type	*equation;	/* where the parsed expression is stored (equation side) */
+int		*np;		/* pointer to the returned parsed expression length */
+char		*cp;		/* string to parse */
+{
+	int	i;
+	char	*cp1, *cp2;
+
+	if (!case_sensitive_flag) {
+		str_tolower(cp);
+	}
+	cp1 = skip_space(cp);
+	if (*cp1 == '#') {
+		cp1++;
+		switch (*cp1) {
+		case '+':
+		case '-':
+			i = decstrtol(cp1, &cp2);
+			i = cur_equation + i;
+			break;
+		default:
+			i = decstrtol(cp1, &cp2) - 1;
+			break;
+		}
+		if (cp1 == cp2 || *cp2) {
+			error(_("Error parsing equation space number."));
+			return NULL;
+		}
+		if (i < 0 || i >= n_equations || n_lhs[i] <= 0) {
+			error(_("No expression available in specified equation space."));
+			return NULL;
+		}
+		if (n_rhs[i]) {
+			blt(equation, rhs[i], n_rhs[i] * sizeof(token_type));
+			*np = n_rhs[i];
+		} else {
+			blt(equation, lhs[i], n_lhs[i] * sizeof(token_type));
+			*np = n_lhs[i];
+		}
+#if	!SILENT
+		list_proc(equation, *np, false);
+		fprintf(gfp, "\n");
+#endif
+		return cp2;
+	}
+	if ((cp = parse_section(equation, np, cp, true)) != NULL) {
+		if (extra_characters(cp))
+			return NULL;
+	}
+	return cp;
+}
+
 /*
  * Parse variable name string pointed to by "cp".
  * The variable name is converted to Mathomatic format and stored in "*vp".
@@ -478,7 +551,7 @@ syntax_error:
  * Return new string position if successful.
  * Display error message and return NULL on failure.
  */
-char	*
+char *
 parse_var(vp, cp)
 long	*vp;
 char	*cp;
@@ -654,10 +727,12 @@ double	*dp;
 {
 	switch (v) {
 	case V_E:
-		*dp = M_E;
+		if (dp)
+			*dp = M_E;
 		return true;
 	case V_PI:
-		*dp = M_PI;
+		if (dp)
+			*dp = M_PI;
 		return true;
 	}
 	return false;
