@@ -1,12 +1,14 @@
 /*
- * Mathomatic simplifying and polynomial routines.
+ * Mathomatic simplifying and general polynomial routines.
  * Includes polynomial and smart division, polynomial factoring, etc.
  * Globals tlhs[] and trhs[] are wiped out by most of these routines.
  *
- * The polynomial division and GCD routines here are not recursive, due to static
- * storage areas.  This limits the polynomial GCD to univariate operation.
+ * The polynomial division and GCD routines here are not recursive, due to static storage areas.
+ * This limits the polynomial GCD routine to mostly univariate operation.
+ * This also does not allow their use during solving.
+ * So far, these limitations have been a good thing, making Mathomatic faster and more stable.
  *
- * Copyright (C) 1987-2008 George Gesslein II.
+ * Copyright (C) 1987-2009 George Gesslein II.
  */
 
 #include "includes.h"
@@ -37,13 +39,15 @@ sort_type	*p1, *p2;
 
 /*
  * Return true if passed expression is strictly a single polynomial term in variable v.
- * The exponent may be anything, as long as it doesn't contain the variable.
+ * The general form of a polynomial term is c*(v^d)
+ * The coefficient (c) and exponent (d) may be any expression,
+ * as long as it doesn't contain the variable v.
  */
 static int
 poly_in_v_sub(p1, n, v, allow_divides)
 token_type	*p1;		/* expression pointer */
 int		n;		/* expression length */
-long		v;		/* variable */
+long		v;		/* Mathomatic variable */
 int		allow_divides;	/* if true, allow division by variable */
 {
 	int	i, k;
@@ -90,14 +94,15 @@ int		allow_divides;	/* if true, allow division by variable */
 }
 
 /*
- * Return true if passed expression is a polynomial in variable v.
- * The exponents may be anything, as long as they don't contain the variable.
+ * Return true if passed expression is a polynomial in the given variable v.
+ * The coefficients and exponents may be anything, as long as they don't contain the variable v.
+ * The passed expression should be fully unfactored, for a proper determination.
  */
 int
 poly_in_v(p1, n, v, allow_divides)
 token_type	*p1;		/* expression pointer */
 int		n;		/* expression length */
-long		v;		/* variable */
+long		v;		/* Mathomatic variable */
 int		allow_divides;	/* allow variable to be right of a divide (negative exponents) as a polynomial term */
 {
 	int	i, j;
@@ -176,8 +181,11 @@ int		*np, loc, level;
  */
 static int
 pf_sub(equation, np, loc, len, level)
-token_type	*equation;
-int		*np, loc, len, level;
+token_type	*equation;	/* equation side holding the possible polynomial to factor */
+int		*np,		/* pointer to length of equation side */
+		loc,		/* index of start of polynomial in equation side */
+		len;		/* length of polynomial */
+int		level;		/* level of additive operators in polynomial */
 {
 	token_type	*p1;
 	int		modified = false, symbolic_modified = false;
@@ -193,14 +201,17 @@ int		*np, loc, len, level;
 	int		vc, cnt;
 	sort_type	va[MAX_VARS];
 	double		d;
+	int		old_partial;
 
 	debug_string(3, "Entering pf_sub().");
+	old_partial = partial_flag;
 	loc2 = loc1 = loc;
 	find_greatest_power(&equation[loc1], len, &v, &d, &j, &k, &div_flag);
 	if (v == 0)
 		return false;
 	blt(save_save, jmp_save, sizeof(jmp_save));
 	if ((i = setjmp(jmp_save)) != 0) {	/* trap errors */
+		partial_flag = old_partial;
 		blt(jmp_save, save_save, sizeof(jmp_save));
 		if (i == 13) {	/* critical error code */
 			longjmp(jmp_save, i);
@@ -212,14 +223,16 @@ int		*np, loc, len, level;
 	for (count = 1;; count++) {
 		blt(trhs, &equation[loc1], len * sizeof(token_type));
 		n_trhs = len;
+		partial_flag = false;
 		uf_simp(trhs, &n_trhs);
+		partial_flag = old_partial;
 		if (level1_plus_count(trhs, n_trhs) < 2) {
 /* must be at least 2 level 1 additive operators to be factorable */
 			goto skip_factor;
 		}
 /* create a variable list with counts of the number of times each variable occurs: */
 		last_v = 0;
-		for (vc = 0; vc < ARR_CNT(va); vc++) {
+		for (vc = 0; vc < ARR_CNT(va);) {
 			cnt = 0;
 			v1 = -1;
 			for (i = 0; i < n_trhs; i += 2) {
@@ -237,8 +250,10 @@ int		*np, loc, len, level;
 			last_v = v1;
 			va[vc].v = v1;
 			va[vc].count = cnt;
+			vc++;
 		}
 		side_debug(3, &equation[loc1], len);
+		side_debug(3, trhs, n_trhs);
 /* find a valid polynomial base variable "v": */
 		cnt = -1;
 		if (v) {
@@ -259,17 +274,40 @@ int		*np, loc, len, level;
 				cnt = va[i].count;
 			}
 		}
-		if (cnt <= 1 || v == 0)
+		if (cnt <= 1)
 			goto skip_factor;
-		blt(tlhs, trhs, n_trhs * sizeof(token_type));
-		n_tlhs = n_trhs;
-		if (!differentiate(tlhs, &n_tlhs, v)) {
-			break;
+		if (v == 0) {
+#if	true
+			goto skip_factor;
+#else
+/* no polynomial variables found, try differentiating with respect to each variable until one works */
+			for (i = 0; i < vc; i++) {
+				if ((va[i].v & VAR_MASK) <= SIGN) {
+					continue;
+				}
+				v = va[i].v;
+				blt(tlhs, trhs, n_trhs * sizeof(token_type));
+				n_tlhs = n_trhs;
+				if (differentiate(tlhs, &n_tlhs, v)) {
+					break;
+				}
+				v = 0;
+			}
+			if (v == 0) {
+				goto skip_factor;
+			}
+#endif
+		} else {
+			blt(tlhs, trhs, n_trhs * sizeof(token_type));
+			n_tlhs = n_trhs;
+			if (!differentiate(tlhs, &n_tlhs, v)) {
+				goto skip_factor;
+			}
 		}
 #if	!SILENT
 		if (debug_level >= 3) {
 			list_var(v, 0);
-			printf(_("Differentiation successful using variable (%s).\n"), var_str);
+			fprintf(gfp, _("Differentiation successful using variable (%s).\n"), var_str);
 		}
 #endif
 		simp_loop(tlhs, &n_tlhs);
@@ -344,9 +382,9 @@ next_v:
 			n_tlhs = k;
 #if	!SILENT
 			if (debug_level >= 3) {
-				printf(_("Trying factor: "));
+				fprintf(gfp, _("Trying factor: "));
 				list_proc(tlhs, n_tlhs, false);
-				printf("\n");
+				fprintf(gfp, "\n");
 			}
 #endif
 			if (poly_gcd(&equation[loc1], len, tlhs, n_tlhs, 0L) == 0)
@@ -520,7 +558,8 @@ long		*vp;
 }
 
 /*
- * Compute polynomial Greatest Common Divisor of "larger" and "smaller".
+ * Compute the polynomial Greatest Common Divisor of the expressions in "larger" and "smaller".
+ * This GCD routine is used for polynomial factoring, etc.
  *
  * Return true if successful.
  * Return the GCD in trhs[].
@@ -562,7 +601,7 @@ long		v;
 		blt(gcd_divisor, tlhs, n_tlhs * sizeof(token_type));
 		len_d = n_tlhs;
 		if (poly_div(larger, llen, gcd_divisor, len_d, &v) != 2) {
-			warning("Polynomial GCD found, but larger divide failed in poly_gcd().");
+			debug_string(1, "Polynomial GCD found, but larger divide failed in poly_gcd().");
 			return 0;
 		}
 	}
@@ -577,7 +616,8 @@ long		v;
 }
 
 /*
- * Compute polynomial Greatest Common Divisor of "larger" and "smaller".
+ * Compute the polynomial Greatest Common Divisor of the expressions in "larger" and "smaller".
+ * This GCD routine is used by division simplifiers.
  *
  * Return true if successful.
  * Return larger/GCD in tlhs[].
@@ -628,7 +668,7 @@ long		v;
 		if (!level1_plus_count(gcd_divisor, len_d))
 			return 0;
 		if (poly_div(smaller, slen, gcd_divisor, len_d, &v) != 2) {
-			warning("Polynomial GCD found, but smaller divide failed in poly2_gcd().");
+			debug_string(1, "Polynomial GCD found, but smaller divide failed in poly2_gcd().");
 			return 0;
 		}
 		blt(trhs, gcd_divisor, len_d * sizeof(token_type));
@@ -640,7 +680,7 @@ long		v;
 		blt(tlhs, trhs, n_trhs * sizeof(token_type));
 		n_tlhs = n_trhs;
 		if (poly_div(larger, llen, tlhs, n_tlhs, &v) != 2) {
-			warning("Polynomial GCD found, but larger divide failed in poly2_gcd().");
+			debug_string(1, "Polynomial GCD found, but larger divide failed in poly2_gcd().");
 			return 0;
 		}
 		blt(trhs, gcd_divisor, len_d * sizeof(token_type));
@@ -654,10 +694,11 @@ long		v;
 }
 
 /*
- * Return true if passed expression is entirely integer.
+ * Return true if passed expression is entirely integer and all variables are
+ * integer like "integer" and "sign".
  *
- * Assuming the variables are integer, the result of evaluating
- * the expression must be integer if this returns true.
+ * The result of evaluating the expression must be integer if this returns true.
+ * Should be unfactored with uf_pplus() first for a proper determination.
  */
 int
 is_integer_expr(p1, n)
@@ -665,12 +706,23 @@ token_type	*p1;	/* expression pointer */
 int		n;	/* length of expression */
 {
 	int	i;
+	long	v;
 
+	parse_var(&v, V_INTEGER_NAME);
 	for (i = 0; i < n; i++) {
-		if ((p1[i].kind == OPERATOR && p1[i].token.operatr == DIVIDE)
-		    || (p1[i].kind == CONSTANT && fmod(p1[i].token.constant, 1.0))
-		    || (p1[i].kind == VARIABLE && p1[i].token.variable <= IMAGINARY)) {
-			return false;
+		switch (p1[i].kind) {
+		case OPERATOR:
+			if (p1[i].token.operatr == DIVIDE)
+				return false;
+			break;
+		case CONSTANT:
+			if (fmod(p1[i].token.constant, 1.0))
+				return false;
+			break;
+		case VARIABLE:
+			if (p1[i].token.variable != v && (p1[i].token.variable & VAR_MASK) != SIGN)
+				return false;
+			break;
 		}
 	}
 	return true;
@@ -678,7 +730,6 @@ int		n;	/* length of expression */
 
 /*
  * This routine is a modulus (%) simplifier for equation sides.
- * It does polynomial division, so tlhs and trhs are wiped out.
  *
  * Return true if expression was modified.
  */
@@ -700,8 +751,8 @@ int		*np, loc, level;
 	int	i1, i2, i3, i4, i5;
 	int	op, last_op2;
 	int	len1, len2, len3;
-	long	v;
 	int	diff_sign;
+	long	v;		/* Mathomatic variable */
 
 	for (i = loc; i < *np && equation[i].level >= level;) {
 		if (equation[i].level > level) {
@@ -818,18 +869,15 @@ int		*np, loc, level;
 					}
 					break;
 				}
-				/* remove integer*n multiples in x for x%n by doing */
-				/* polynomial division and replacing with remainder%n */
+				/* Remove integer*n multiples in x for x%n by doing */
+				/* polynomial division x/n and replacing with remainder%n. */
+				/* Globals tlhs[] and trhs[] are wiped out by the polynomial division here. */
 				v = 0;
 				if (poly_div(&equation[j], len2, &equation[i+1], len1, &v)) {
+					uf_pplus(tlhs, &n_tlhs);	/* so integer%(integer^integer) isn't simplified to 0 */
 					if (is_integer_expr(tlhs, n_tlhs)) {
 						if ((*np + (n_trhs - len2)) > n_tokens)
 							error_huge();
-#if	!SILENT
-						if (var_count(tlhs, n_tlhs)) {
-							printf(_("Modulus simplification made some variables integer only.\n"));
-						}
-#endif
 						for (k = 0; k < n_trhs; k++)
 							trhs[k].level += level;
 						blt(&equation[j+n_trhs], &equation[j+len2], (*np - (j + len2)) * sizeof(token_type));
@@ -846,8 +894,10 @@ int		*np, loc, level;
 
 /*
  * This routine is a division simplifier for equation sides.
+ * It reduces algebraic fractions.
  *
- * Return true if a Greatest Common Divisor was found and
+ * Return true if any polynomial Greatest Common Divisor (GCD)
+ * between a numerator and denominator was found and
  * the expression was simplified by dividing the numerator
  * and denominator by the GCD.
  */
@@ -919,7 +969,7 @@ store_code:
 					blt(&equation[i+n_tlhs+1], &equation[i+1+len1], (*np - (i + 1 + len1)) * sizeof(token_type));
 					*np += n_tlhs - len1;
 					blt(&equation[i+1], tlhs, n_tlhs * sizeof(token_type));
-					debug_string(0, _("Division simplified with polynomial GCD."));
+					debug_string(1, _("Division simplified with polynomial GCD."));
 					modified = true;
 					goto start;
 				}
@@ -951,7 +1001,7 @@ int		*np;
 int		poly_flag;
 int		quick_flag;
 {
-	int	rv;
+	int	rv = false;
 
 	if (quick_flag)
 		group_proc(equation, np);
@@ -971,7 +1021,7 @@ int		*np, loc, level, code;
 	int	op, op2, last_op2;
 	int	len1, len2, real_len1;
 	long	v;
-	int	rv;
+	int	rv = 0;
 	int	flag, power_flag, zero_remainder;
 
 	for (i = loc + 1; i < *np && equation[i].level >= level; i += 2) {
@@ -1061,12 +1111,12 @@ next_thingy:
 						tlhs[k].level += 2;
 					side_debug(3, &equation[j], len2);
 					side_debug(3, &equation[i+1], len1);
-					simpb_side(tlhs, &n_tlhs, true, 3);
+					simpb_side(tlhs, &n_tlhs, false, true, 3);	/* parameters should match what's used in simpa_side() */
 					side_debug(3, tlhs, n_tlhs);
 					if (power_flag) {
 						k = (var_count(tlhs, n_tlhs) <= var_count(&equation[j], len2));
 					} else {
-						k = (var_count(tlhs, n_tlhs) <= (var_count(&equation[j], len2) + var_count(&equation[i+1], len1)));
+						k = ((var_count(tlhs, n_tlhs) + (n_tlhs >= len1 + 1 + len2)) <= (var_count(&equation[j], len2) + var_count(&equation[i+1], len1)));
 					}
 					if (k) {
 						for (k = 0; k < n_tlhs; k++)
@@ -1150,14 +1200,15 @@ token_type	*d2;		/* pointer to divisor */
 int		len2;		/* length of divisor */
 long		*vp;		/* variable pointer to base variable */
 {
-	int		i, j;
+	int		i;
+	int		rv;
 	int		old_partial;
 	jmp_buf		save_save;
 
 	old_partial = partial_flag;
-	partial_flag = false;	/* We want full unfactoring. */
+	partial_flag = false;	/* We want full unfactoring during polynomial division. */
 	blt(save_save, jmp_save, sizeof(jmp_save));
-	if ((i = setjmp(jmp_save)) != 0) {	/* trap errors */
+	if ((i = setjmp(jmp_save)) != 0) {	/* Trap errors so we almost always return normally. */
 		blt(jmp_save, save_save, sizeof(jmp_save));
 		partial_flag = old_partial;
 		if (i == 13) {	/* critical error code */
@@ -1165,10 +1216,10 @@ long		*vp;		/* variable pointer to base variable */
 		}
 		return false;
 	}
-	j = poly_div_sub(d1, len1, d2, len2, vp);
+	rv = poly_div_sub(d1, len1, d2, len2, vp);
 	blt(jmp_save, save_save, sizeof(jmp_save));
 	partial_flag = old_partial;
-	return j;
+	return rv;
 }
 
 /*
@@ -1214,7 +1265,7 @@ long		*vp;
 	/* Display debugging info. */
 	if (debug_level >= 3) {
 		list_var(*vp, 0);
-		printf(_("poly_div() starts using base variable (%s):\n"), var_str);
+		fprintf(gfp, _("poly_div() starts using base variable (%s):\n"), var_str);
 		side_debug(3, trhs, n_trhs);
 		side_debug(3, tlhs, n_tlhs);
 	}
@@ -1349,7 +1400,7 @@ long		*vp;
  * Do smart division.
  *
  * Smart division is heuristic division much like polynomial division,
- * however instead of basing the division on the highest powers,
+ * however instead of basing the division on the highest powers of a base variable,
  * every term in the dividend is tried, and if a trial makes the
  * expression smaller, we go with that.
  *
@@ -1680,7 +1731,7 @@ long		*vp1;		/* variable pointer */
 	int		count1, count2;
 
 	last_v = 0;
-	for (vc = 0; vc < ARR_CNT(va); vc++) {
+	for (vc = 0; vc < ARR_CNT(va);) {
 		cnt = 0;
 		v1 = -1;
 		for (i = 0; i < n1; i += 2) {
@@ -1698,6 +1749,7 @@ long		*vp1;		/* variable pointer */
 		last_v = v1;
 		va[vc].v = v1;
 		va[vc].count = cnt;
+		vc++;
 	}
 	if (vc <= 0)
 		return 0;
