@@ -1,17 +1,19 @@
 /*
  * Mathomatic simplifying routines.
  *
- * Copyright (C) 1987-2008 George Gesslein II.
+ * Copyright (C) 1987-2009 George Gesslein II.
  */
 
 #include "includes.h"
 
 /*
- * The following defines the maximum number of terms on the same level that can be compared.
- * Most of the stack usage is related this value,
- * this many pointers are pushed onto the stack with each level of parentheses while comparing two expressions.
+ * MAX_COMPARE_TERMS defines the maximum number of terms
+ * on the same level of parentheses that can be compared.
+ * Most of the stack usage is related this value.
+ * Space for this many pointers is reserved on the stack
+ * with each level of parentheses while comparing two expressions.
  */
-#define	MAX_COMPARE_TERMS	(DEFAULT_N_TOKENS / 10)
+#define	MAX_COMPARE_TERMS	(DEFAULT_N_TOKENS / 6)
 
 static int	org_recurse();
 static int	const_recurse();
@@ -30,17 +32,15 @@ token_type	*equation;	/* equation side pointer */
 int		*np;		/* pointer to length of equation side */
 {
 	if (*np <= 0 || (*np & 1) != 1) {
-		error("Internal error: Organize called with bad expression size.");
-		longjmp(jmp_save, 13);
+		error_bug("Internal error: organize() called with bad expression size.");
 	}
 	if (*np > n_tokens) {
-		error("Internal error: Unchecked expression array overflow detected.");
-		longjmp(jmp_save, 13);
+		error_bug("Internal error: expression array overflow detected in organize().");
 	}
 	org_recurse(equation, np, 0, 1, NULL);
 }
 
-static inline void
+static void
 org_up_level(bp, ep, level, invert)
 token_type	*bp, *ep;
 int		level, invert;
@@ -145,7 +145,7 @@ int		loc, level, *elocp;
 }
 
 /*
- * The most basic simplification loop.
+ * The quickest, most basic simplification loop.
  * Just does constant simplification.
  */
 void
@@ -174,6 +174,38 @@ int		*np;		/* pointer to length of equation side */
 	side_debug(5, equation, *np);
 }
 
+/*
+ * Configurable high level simplify routine.
+ */
+void
+simp_ssub(equation, np, v, d, power_flag, times_flag, fc_level)
+token_type	*equation;	/* pointer to the beginning of equation side to simplify */
+int		*np;		/* pointer to length of equation side */
+long		v;		/* variable to factor, 0L or MATCH_ANY to factor all variables */
+double		d;		/* factor expressions raised to the power of this if v */
+int		power_flag;	/* factor_power() flag */
+int		times_flag;	/* factor_times() flag */
+int		fc_level;	/* factor constants code, passed to factor_constants() */
+{
+	do {
+		do {
+			do {
+				do {
+					do {
+						do {
+							do {
+								do {
+									elim_loop(equation, np);
+								} while (simp2_power(equation, np));
+							} while (times_flag && factor_times(equation, np));
+						} while (elim_sign(equation, np));
+					} while (subtract_itself(equation, np));
+				} while (factor_constants(equation, np, fc_level));
+			} while (factor_divide(equation, np, v, d));
+		} while (factor_plus(equation, np, v, d));
+	} while (power_flag && factor_power(equation, np));
+}
+
 /*
  * Quickly and very basically simplify an equation space.
  * No factoring is done.
@@ -198,7 +230,7 @@ simp_side(equation, np)
 token_type	*equation;	/* pointer to the beginning of equation side to simplify */
 int		*np;		/* pointer to length of equation side */
 {
-	simp_ssub(equation, np, 0L, 0.0, true, true, 1);
+	simp_ssub(equation, np, 0L, 1.0, true, true, 2 /* was 1 */);
 }
 
 /*
@@ -232,18 +264,25 @@ long		v;		/* variable to factor, 0 for all variables */
 }
 
 /*
- * Convert expression with any algebraic fractions into a single fraction.
+ * Factor only the specified variable "v".
+ * If v is IMAGINARY, do complex number simplification.
  */
 void
-frac_side(equation, np)
-token_type	*equation;	/* pointer to the beginning of equation side */
-int		*np;		/* pointer to length of equation side */
+factorv(equation, np, v)
+token_type	*equation;	/* pointer to the beginning of equation side to factor */
+int		*np;		/* pointer to equation side length */
+long		v;		/* variable to factor */
 {
-	if (*np == 0)
-		return;
+	if (v == IMAGINARY) {
+		do {
+			elim_loop(equation, np);
+		} while (complex_root_simp(equation, np));
+	}
 	do {
-		simpb_side(equation, np, false, 2);
-	} while (super_factor(equation, np, 2));
+		do {
+			simp_loop(equation, np);
+		} while (factor_plus(equation, np, v, 0.0));
+	} while (v == IMAGINARY && div_imaginary(equation, np));
 }
 
 /*
@@ -265,7 +304,8 @@ int		*np;
 	uf_simp(equation, np);
 	factorv(equation, np, IMAGINARY);
 	simp_side(equation, np);
-	uf_simp(equation, np);
+	make_fractions(equation, np);
+	uf_tsimp(equation, np);
 	approximate_roots = false;
 }
 
@@ -317,143 +357,49 @@ int		*np;
 }
 
 /*
- * Main simplify code used by the simplify command.
- * This is the slowest and most thorough simplify of all.
- * Applies many equivalent algebraic transformations and their inverses (like unfactor and factor),
- * then does generalized polynomial simplifications.
- *
- * Globals tlhs[] and trhs[] are wiped out.
+ * Combine all like denominators.
  */
 void
-simpa_side(equation, np, quick_flag, frac_flag)
-token_type	*equation;	/* pointer to the beginning of equation side to simplify */
-int		*np;		/* pointer to length of the equation side */
-int		quick_flag;	/* "simplify quick" option, simpler fractions with no (x+1)^2 expansion */
-int		frac_flag;	/* "simplify fraction" option, simplify to the ratio of two polynomials */
+simp_divide(equation, np)
+token_type	*equation;
+int		*np;
 {
-	int		i;
-	int		flag, poly_flag = true;
-	jmp_buf		save_save;
-
-	if (*np <= 1)	/* no need to simplify a single constant or variable */
-		return;
-	simp_loop(equation, np);
-	poly_factor(equation, np);
-	simpb_side(equation, np, true, 1);
-	simp_ssub(equation, np, 0L, 0.0, true, true, 1);
-	rationalize(equation, np);
-	unsimp_power(equation, np);
-	uf_times(equation, np);
-	simp_loop(equation, np);
-	/* Here is the only place in Mathomatic that we do modulus (%) simplification: */
-	uf_repeat(equation, np);
 	do {
-		elim_loop(equation, np);
-	} while (mod_simp(equation, np));
-	/* Here we try to simplify out unnecessary negative constants and imaginary numbers: */
-	simp_i(equation, np);
-	unsimp_power(equation, np);
-	uf_times(equation, np);
-	simp_ssub(equation, np, 0L, 1.0, true, true, 1);
-	unsimp_power(equation, np);
-	uf_neg_help(equation, np);
-	uf_times(equation, np);
+		do {
+			simp_loop(equation, np);
+		} while (factor_constants(equation, np, 1));
+	} while (factor_divide(equation, np, 0L, 0.0));
+}
+
+/*
+ * Combine all like denominators containing variable "v".
+ * Don't call factor_times().
+ */
+void
+simp2_divide(equation, np, v, fc_level)
+token_type	*equation;
+int		*np;
+long		v;
+int		fc_level;
+{
 	do {
 		do {
 			do {
-				simp_ssub(equation, np, 0L, 1.0, false, true, 1);
-			} while (poly_gcd_simp(equation, np));
-		} while (uf_power(equation, np));
-	} while (super_factor(equation, np, 2));
-	unsimp_power(equation, np);
-	uf_times(equation, np);
-	factorv(equation, np, IMAGINARY);
-	uf_pplus(equation, np);
-	simp_ssub(equation, np, 0L, 1.0, true, false, 1);
-	uf_times(equation, np);
-	uf_pplus(equation, np);
-	factorv(equation, np, IMAGINARY);
-	uf_power(equation, np);
-	do {
-		do {
-			simp_ssub(equation, np, 0L, 1.0, false, true, 1);
-		} while (uf_power(equation, np));
-	} while (super_factor(equation, np, 2));
-	if (quick_flag) {
-do_quick:
-		uf_tsimp(equation, np);
-	} else {
-		n_tlhs = *np;
-		blt(tlhs, equation, n_tlhs * sizeof(token_type));
-		blt(save_save, jmp_save, sizeof(jmp_save));
-		if ((i = setjmp(jmp_save)) != 0) {	/* trap errors */
-			blt(jmp_save, save_save, sizeof(jmp_save));
-			if (i == 13) {	/* critical error code */
-				longjmp(jmp_save, i);
-			}
-			/* an error occurred, restore the original expression */
-			*np = n_tlhs;
-			blt(equation, tlhs, n_tlhs * sizeof(token_type));
-			goto do_quick;	/* skip power expansion */
-		}
-		/* expand powers of 2 and higher, might result in error_huge() trap */
-		do {
-			uf_repeat(equation, np);
-		} while (uf_tsimp(equation, np));
-		blt(jmp_save, save_save, sizeof(jmp_save));
-	}
-	simpb_side(equation, np, true, 1);
-	debug_string(1, "Simplify result before applying polynomial operations:");
-	side_debug(1, equation, *np);
-	for (flag = false;;) {
-		/* divide top and bottom of fractions by any polynomial GCD found */
-		if (poly_gcd_simp(equation, np)) {
-			flag = false;
-			simpb_side(equation, np, true, 1);
-		}
-		if (!flag) {
-			/* factor polynomials */
-			if (poly_factor(equation, np)) {
-				flag = true;
-				simpb_side(equation, np, true, 1);
-				continue;
-			}
-		}
-		/* simplify algebraic fractions with polynomial and smart division */
-		if (!frac_flag && div_remainder(equation, np, poly_flag, quick_flag)) {
-			flag = false;
-			simpb_side(equation, np, true, 1);
-			continue;
-		}
-		break;
-	}
-	simpb_side(equation, np, true, 1);
-	simp_constant_power(equation, np);
-	simp_ssub(equation, np, 0L, 1.0, true, true, 1);
-	unsimp_power(equation, np);
-	make_fractions(equation, np);
-	factor_power(equation, np);
-	partial_flag = frac_flag;
-	uf_tsimp(equation, np);
-	partial_flag = true;
-	make_fractions(equation, np);
-	uf_power(equation, np);
-	integer_root_simp(equation, np);
-	simpb_side(equation, np, true, 3);
-	if (!frac_flag) {
-		while (div_remainder(equation, np, poly_flag, quick_flag)) {
-			simpb_side(equation, np, true, 3);
-		}
-	}
-	poly_factor(equation, np);
-	simpb_side(equation, np, true, 2);
+				do {
+					do {
+						elim_loop(equation, np);
+					} while (simp2_power(equation, np));
+				} while (elim_sign(equation, np));
+			} while (subtract_itself(equation, np));
+		} while (factor_constants(equation, np, fc_level));
+	} while (factor_divide(equation, np, v, 0.0));
 }
 
 /*
  * Compare function for qsort(3) within simpb_side().
  */
 static int
-vcmp(p1, p2)
+simpb_vcmp(p1, p2)
 sort_type	*p1, *p2;
 {
 	if (((p1->v & VAR_MASK) == SIGN) == ((p2->v & VAR_MASK) == SIGN)) {
@@ -475,25 +421,29 @@ sort_type	*p1, *p2;
 }
 
 /*
- * Beauty simplifier.
- * Neat simplify and factor routine.
+ * Beauty simplifier for equation sides.
+ * This is the neat simplify and variable factor routine, to make a pleasing display.
  * Factors variables in order: "sign" variables first, then by frequency.
  */
-simpb_side(equation, np, power_flag, fc_level)
+void
+simpb_side(equation, np, uf_power_flag, power_flag, fc_level)
 token_type	*equation;	/* pointer to the beginning of equation side */
 int		*np;		/* pointer to length of equation side */
+int		uf_power_flag;	/* uf_allpower() flag */
 int		power_flag;	/* factor_power() flag */
 int		fc_level;	/* factor constants code, passed to factor_constants() */
 {
 	int		i;
-	int		vc, cnt;
-	long		v1, last_v;
-	sort_type	va[MAX_VARS];
+	int		vc, cnt;	/* counts */
+	long		v1, last_v;	/* Mathomatic variables */
+	sort_type	va[MAX_VARS];	/* array of all real variables found in equation side */
 
 	simp_loop(equation, np);
-	uf_allpower(equation, np);
+	if (uf_power_flag) {
+		uf_allpower(equation, np);
+	}
 	last_v = 0;
-	for (vc = 0; vc < ARR_CNT(va); vc++) {
+	for (vc = 0; vc < ARR_CNT(va);) {
 		cnt = 0;
 		v1 = -1;
 		for (i = 0; i < *np; i += 2) {
@@ -509,22 +459,29 @@ int		fc_level;	/* factor constants code, passed to factor_constants() */
 		if (v1 == -1)
 			break;
 		last_v = v1;
-		va[vc].v = v1;
-		va[vc].count = cnt;
+		if (v1 > IMAGINARY) {	/* ignore constant variables */
+			va[vc].v = v1;
+			va[vc].count = cnt;
+			vc++;
+		}
 	}
 	if (vc) {
-		qsort((char *) va, vc, sizeof(*va), vcmp);
+		/* sort variable array va[] */
+		qsort((char *) va, vc, sizeof(*va), simpb_vcmp);
+		/* factor division by most frequently occurring variables first */
 		simp2_divide(equation, np, va[0].v, fc_level);
 		for (i = 1; i < vc; i++) {
 			if (factor_divide(equation, np, va[i].v, 0.0))
 				simp2_divide(equation, np, va[i].v, fc_level);
 		}
 		simp2_divide(equation, np, 0L, fc_level);
+		/* factor all subexpressions in order of most frequently occurring variables */
 		for (i = 0; i < vc; i++) {
 			while (factor_plus(equation, np, va[i].v, 0.0)) {
 				simp2_divide(equation, np, 0L, fc_level);
 			}
 		}
+		/* make sure equation side is completely factored */
 		while (factor_plus(equation, np, MATCH_ANY, 0.0)) {
 			simp2_divide(equation, np, 0L, fc_level);
 		}
@@ -533,40 +490,172 @@ int		fc_level;	/* factor constants code, passed to factor_constants() */
 }
 
 /*
- * Configurable high level simplify routine.
+ * Convert expressions with any algebraic fractions into a single simple fraction.
+ * Used by the fraction command.
+ *
+ * Globals tlhs[] and trhs[] are wiped out.
  */
-simp_ssub(equation, np, v, d, power_flag, times_flag, fc_level)
-token_type	*equation;	/* pointer to the beginning of equation side to simplify */
+void
+simple_frac_side(equation, np)
+token_type	*equation;	/* pointer to the beginning of equation side */
 int		*np;		/* pointer to length of equation side */
-long		v;		/* variable to factor, 0L or MATCH_ANY to factor all variables */
-double		d;		/* factor expressions raised to the power of this if v */
-int		power_flag;	/* factor_power() flag */
-int		times_flag;	/* factor_times() flag */
-int		fc_level;	/* factor constants code, passed to factor_constants() */
 {
+	if (*np == 0)
+		return;
 	do {
 		do {
 			do {
-				do {
-					do {
-						do {
-							do {
-								do {
-									elim_loop(equation, np);
-								} while (simp2_power(equation, np));
-							} while (times_flag && factor_times(equation, np));
-						} while (elim_sign(equation, np));
-					} while (subtract_itself(equation, np));
-				} while (factor_constants(equation, np, fc_level));
-			} while (factor_divide(equation, np, v, d));
-		} while (factor_plus(equation, np, v, d));
-	} while (power_flag && factor_power(equation, np));
+				simp_ssub(equation, np, 0L, 1.0, false, true, 1);
+			} while (poly_gcd_simp(equation, np));
+		} while (uf_power(equation, np));
+	} while (super_factor(equation, np, 2));
+	make_fractions(equation, np);
+	uf_tsimp(equation, np);
+	poly_factor(equation, np);
+	simpb_side(equation, np, true, false, 2);
+}
+
+/*
+ * Main simplify code used by the simplify command.
+ * This is the slowest and most thorough simplify of all.
+ * Applies many equivalent algebraic transformations and their inverses (like unfactor and factor),
+ * then does generalized polynomial simplifications.
+ *
+ * Globals tlhs[] and trhs[] are wiped out.
+ */
+void
+simpa_side(equation, np, quick_flag, frac_flag)
+token_type	*equation;	/* pointer to the beginning of equation side to simplify */
+int		*np;		/* pointer to length of the equation side */
+int		quick_flag;	/* "simplify quick" option, simpler fractions with no (x+1)^2 expansion */
+int		frac_flag;	/* "simplify fraction" option, simplify to the ratio of two polynomials */
+{
+	int		i;
+	int		flag, poly_flag = true;
+	jmp_buf		save_save;
+
+	if (*np <= 1)	/* no need to simplify a single constant or variable */
+		return;
+	side_debug(2, equation, *np);
+	simpb_side(equation, np, false, true, 1);
+	if (rationalize_denominators) {
+		rationalize(equation, np);
+	}
+	unsimp_power(equation, np);
+	uf_times(equation, np);
+	simp_loop(equation, np);
+
+/* Here is the only place in Mathomatic that we do modulus (%) simplification: */
+	uf_pplus(equation, np);
+	uf_repeat(equation, np);
+	do {
+		elim_loop(equation, np);
+	} while (mod_simp(equation, np));
+
+/* Here we try to simplify out unnecessary negative constants and imaginary numbers: */
+	simp_i(equation, np);
+	unsimp_power(equation, np);
+	uf_times(equation, np);
+	simp_ssub(equation, np, 0L, 1.0, true, true, 1);
+	unsimp_power(equation, np);
+	uf_neg_help(equation, np);
+	uf_times(equation, np);
+	do {
+		do {
+			do {
+				simp_ssub(equation, np, 0L, 1.0, false, true, 2);
+			} while (poly_gcd_simp(equation, np));
+		} while (uf_power(equation, np));
+	} while (super_factor(equation, np, 2));
+	unsimp_power(equation, np);
+	uf_times(equation, np);
+	factorv(equation, np, IMAGINARY);
+	uf_pplus(equation, np);
+	simp_ssub(equation, np, 0L, 1.0, true, false, 1);
+	uf_times(equation, np);
+	uf_pplus(equation, np);
+	factorv(equation, np, IMAGINARY);
+	uf_power(equation, np);
+	do {
+		do {
+			do {
+				simp_ssub(equation, np, 0L, 1.0, false, true, 2);
+			} while (poly_gcd_simp(equation, np));
+		} while (uf_power(equation, np));
+	} while (super_factor(equation, np, 2));
+
+/* Here we do the greatest expansion; if it fails, do less expansion. */
+	partial_flag = frac_flag;
+	n_tlhs = *np;
+	blt(tlhs, equation, n_tlhs * sizeof(token_type));
+	blt(save_save, jmp_save, sizeof(jmp_save));
+	if ((i = setjmp(jmp_save)) != 0) {	/* trap errors */
+		blt(jmp_save, save_save, sizeof(jmp_save));
+		if (i == 13) {	/* critical error code */
+			longjmp(jmp_save, i);
+		}
+		/* an error occurred, restore the original expression */
+		*np = n_tlhs;
+		blt(equation, tlhs, n_tlhs * sizeof(token_type));
+		debug_string(1, "Simplify not expanding fully, due to oversized expression.");
+		partial_flag = true;	/* expand less */
+		uf_tsimp(equation, np);
+	} else {
+		if (quick_flag) {
+			uf_tsimp(equation, np);
+		} else {
+			/* expand powers of 2 and higher, might result in error_huge() trap */
+			do {
+				uf_power(equation, np);
+				uf_repeat(equation, np);
+			} while (uf_tsimp(equation, np));
+		}
+		blt(jmp_save, save_save, sizeof(jmp_save));
+	}
+	partial_flag = true;
+
+	simpb_side(equation, np, true, true, 2);
+	debug_string(1, "Simplify result before applying polynomial operations:");
+	side_debug(1, equation, *np);
+	for (flag = false;;) {
+		/* divide top and bottom of fractions by any polynomial GCD found */
+		if (poly_gcd_simp(equation, np)) {
+			flag = false;
+			simpb_side(equation, np, false, true, 3);
+		}
+		/* factor polynomials */
+		if (!flag && poly_factor(equation, np)) {
+			flag = true;
+			simpb_side(equation, np, false, true, 3);
+			continue;
+		}
+		/* simplify algebraic fractions with polynomial and smart division */
+		if (!frac_flag && div_remainder(equation, np, poly_flag, quick_flag)) {
+			flag = false;
+			simpb_side(equation, np, false, true, 3);
+			continue;
+		}
+		break;
+	}
+	simpb_side(equation, np, true, true, 1);
+	simp_constant_power(equation, np);
+	simp_loop(equation, np);
+	unsimp_power(equation, np);
+	make_fractions(equation, np);
+	factor_power(equation, np);
+	uf_tsimp(equation, np);
+	make_fractions(equation, np);
+	uf_power(equation, np);
+	integer_root_simp(equation, np);
+	simpb_side(equation, np, true, true, 3);
+	poly_factor(equation, np);
+	simpb_side(equation, np, true, !frac_flag, 2);
 }
 
 /*
  * Commonly used quick simplify routine that doesn't factor.
  *
- * Return true if factor_times() simplified something.
+ * Return true if factor_times() did something.
  */
 int
 simp_loop(equation, np)
@@ -592,65 +681,12 @@ int		*np;		/* pointer to length of equation side */
 }
 
 /*
- * Factor only the specified variable "v".
- * If v is IMAGINARY, do complex number simplification.
- */
-factorv(equation, np, v)
-token_type	*equation;	/* pointer to the beginning of equation side to factor */
-int		*np;		/* pointer to equation side length */
-long		v;		/* variable to factor */
-{
-	if (v == IMAGINARY) {
-		do {
-			elim_loop(equation, np);
-		} while (complex_root_simp(equation, np));
-	}
-	do {
-		do {
-			simp_loop(equation, np);
-		} while (factor_plus(equation, np, v, 0.0));
-	} while (v == IMAGINARY && div_imaginary(equation, np));
-}
-
-/*
- * Combine all like denominators.
- */
-simp_divide(equation, np)
-token_type	*equation;
-int		*np;
-{
-	do {
-		do {
-			simp_loop(equation, np);
-		} while (factor_constants(equation, np, 1));
-	} while (factor_divide(equation, np, 0L, 0.0));
-}
-
-/*
- * Combine all like denominators containing variable "v".
- * Don't call factor_times().
- */
-simp2_divide(equation, np, v, fc_level)
-token_type	*equation;
-int		*np;
-long		v;
-int		fc_level;
-{
-	do {
-		do {
-			do {
-				do {
-					do {
-						elim_loop(equation, np);
-					} while (simp2_power(equation, np));
-				} while (elim_sign(equation, np));
-			} while (subtract_itself(equation, np));
-		} while (factor_constants(equation, np, fc_level));
-	} while (factor_divide(equation, np, v, 0.0));
-}
-
-/*
- * Convert (x^n)^m to x^(n*m).
+ * Convert (x^n)^m to x^(n*m) when appropriate.
+ * Recently fixed to work the same as Maxima when symb_flag is false,
+ * otherwise always converts (x^n)^m to x^(n*m),
+ * which sometimes simplifies more.
+ * Earlier versions of Mathomatic always converted (x^n)^m to x^(n*m),
+ * which is the wrong thing to do, because they are not always equivalent.
  *
  * Return true if expression was modified.
  */
@@ -660,52 +696,52 @@ token_type	*equation;
 int		*np;
 {
 	int		i, j, k;
-	int		level;
-	int		left_level;
-	int		diff;
+	int		ilevel, jlevel;
 	int		modified = false;
 	double		numerator, denominator;
 
 	for (i = 1; i < *np; i += 2) {
 		if (equation[i].token.operatr != POWER)
 			continue;
-		level = equation[i].level;
-		left_level = 1;
-		for (j = i - 2; j >= 0; j -= 2) {
-			if (equation[j].level < level) {
-				left_level = equation[j].level;
-				break;
-			}
-		}
+		ilevel = equation[i].level;
 		for (j = i + 2; j < *np; j += 2) {
-			if (equation[j].level <= level) {
-				if (left_level <= equation[j].level && equation[j].token.operatr == POWER) {
-					if (!symb_flag) {
-						if ((i + 2) == j && equation[i+1].kind == CONSTANT
-						    && equation[j].level == equation[j+1].level
-						    && equation[j+1].kind == CONSTANT) {
-							f_to_fraction(equation[i+1].token.constant, &numerator, &denominator);
-							if (fmod(numerator, 2.0) == 0.0) {
-								f_to_fraction(equation[j+1].token.constant, &numerator, &denominator);
-								if (fmod(denominator, 2.0) == 0.0) {
+			jlevel = equation[j].level;
+			if (jlevel == ilevel - 1 && equation[j].token.operatr == POWER) {
+				if (!symb_flag) {
+					if (jlevel == equation[j+1].level
+					    && equation[j+1].kind == CONSTANT) {
+						f_to_fraction(equation[j+1].token.constant, &numerator, &denominator);
+						if (fmod(denominator, 2.0) == 0.0) {
+							if ((i + 2) == j && equation[i+1].kind == CONSTANT) {
+								f_to_fraction(equation[i+1].token.constant, &numerator, &denominator);
+								if (fmod(numerator, 2.0) == 0.0) {
 									break;
 								}
+							} else
+								break;
+						}
+					} else {
+						if ((i + 2) == j && equation[i+1].kind == CONSTANT) {
+							f_to_fraction(equation[i+1].token.constant, &numerator, &denominator);
+							if (fmod(numerator, 2.0) == 0.0) {
+								break;
 							}
 						} else
 							break;
 					}
-					equation[j].token.operatr = TIMES;
-					for (k = i + 1; k < j; k++)
-						equation[k].level++;
-					diff = (level - equation[j].level) + 1;
-					level = equation[j].level;
-					for (k = j; k < *np && equation[k].level >= level; k++) {
-						equation[k].level += diff;
-					}
-					modified = true;
 				}
+				equation[j].token.operatr = TIMES;
+				for (k = j; k < *np && equation[k].level >= jlevel; k++) {
+					equation[k].level += 2;
+				}
+				for (k = i + 1; k < j; k++)
+					equation[k].level++;
+				i -= 2;
+				modified = true;
 				break;
 			}
+			if (jlevel <= ilevel)
+				break;
 		}
 	}
 	return modified;
@@ -830,7 +866,9 @@ int		*np, i;
 }
 
 /*
- * Simplify constant^(constant*x).
+ * Simplify constant^(constant*x) to (constant^constant)^x.
+ * I am not sure if this is a good thing, because it changes the base of the exponent.
+ * The tests don't require this routine.
  *
  * Return true if equation side was modified.
  */
@@ -842,9 +880,11 @@ int		*np;
 	int	modified = false;
 	int	i;
 
-	for (i = 1; i < *np; i += 2) {
-		if (equation[i].token.operatr == POWER) {
-			modified |= sp2_sub(equation, np, i);
+	if (symb_flag) {
+		for (i = 1; i < *np; i += 2) {
+			if (equation[i].token.operatr == POWER) {
+				modified |= sp2_sub(equation, np, i);
+			}
 		}
 	}
 	return modified;
@@ -944,20 +984,25 @@ int		iflag;		/* produce imaginary numbers flag */
  * Do the floating point arithmetic for Mathomatic.
  *
  * Return true if successful.
+ * domain_check must be set to false after this.
  */
-static int
+int
 calc(op1p, k1p, op2, k2)
-int	*op1p;	/* pointer to operator 1, sometimes modified */
-double	*k1p;	/* pointer to operand 1 and where to store the result */
-int	op2;	/* operator 2 */
-double	k2;	/* operand 2 */
+int	*op1p;	/* Pointer to operator 1, which comes immediately before operand 1 if this operator exists. */
+double	*k1p;	/* Pointer to operand 1 and where to store the result of the calculation. */
+int	op2;	/* Operator 2; always exists and comes immediately before operand 2, and usually after operand 1. */
+double	k2;	/* Operand 2; ignored for unary operators. */
 {
 	int	op1;
 	double	d, d1, d2;
 
 	domain_check = false;
 	errno = 0;
-	op1 = *op1p;
+	if (op1p) {
+		op1 = *op1p;
+	} else {
+		op1 = 0;
+	}
 	switch (op2) {
 	case PLUS:
 	case MINUS:
@@ -1016,25 +1061,31 @@ double	k2;	/* operand 2 */
 		/* Another way to get the remainder of division: */
 		*k1p = modf(*k1p / k2, &d) * k2;
 #endif
-		if (true_modulus && *k1p < 0.0) {
+		if (modulus_mode && *k1p < 0.0) {
 			*k1p += fabs(k2);	/* make positive */
 		}
+		if (modulus_mode == 1 && k2 < 0.0 && *k1p > 0.0) {
+			*k1p += k2;		/* make negative */
+		}
+		/* modulus_mode == 0 result same sign as dividend,
+		   modulus_mode == 1 result same sign as divisor,
+                   modulus_mode == 2 result is always positive or zero */
 		break;
 	case POWER:
 		if (*k1p < 0.0 && fmod(k2, 1.0) != 0.0) {
+			/* it's probably imaginary; pow() will give a domain error, so skip these calculations */
 			break;
 		}
-		if (*k1p == 0.0 && k2 < 0.0) {
-			error(_("Divide by zero error (0 raised to negative power)."));
-			longjmp(jmp_save, 2);
-		}
 		domain_check = true;
 		if (*k1p == 0.0 && k2 == 0.0) {
-			warning(_("0^0 encountered, might be considered undefined."));
-			d = 1.0;	/* some people don't know 0^0 */
+			warning(_("0^0 encountered, might be considered indeterminate."));
+			d = 1.0;	/* 0^0 = 1 */
+		} else if (*k1p == 0 && k2 < 0.0) {
+			warning(_("Divide by zero (0 raised to negative power)."));
+			d = INFINITY;
 		} else {
 			d = pow(*k1p, k2);
-			if (preserve_roots && !approximate_roots) {
+			if (preserve_surds && !approximate_roots) {
 				if (isfinite(k2) && fmod(k2, 1.0) != 0.0 && f_to_fraction(*k1p, &d1, &d2)) {
 					if (!f_to_fraction(d, &d1, &d2)) {
 						domain_check = false;
@@ -1077,9 +1128,9 @@ int		*np, loc, level, iflag;
 {
 	int		loc1, old_loc;
 	int		const_count = 0;
-	int		op, zero = 0;
+	int		op;
 	int		modified = false;
-	double		d, d1, d2, numerator, denominator;
+	double		d1, d2, d3, numerator, denominator;
 	complexs	cv, p;
 
 	loc1 = old_loc = loc;
@@ -1103,14 +1154,14 @@ beginning:
 				continue;
 			}
 			op = equation[loc-1].token.operatr;
-			d = equation[loc1].token.constant;
+			d1 = equation[loc1].token.constant;
 			d2 = equation[loc].token.constant;
-			if (calc((loc1 <= old_loc) ? &zero : &equation[loc1-1].token.operatr, &d, op, d2)) {
+			if (calc((loc1 <= old_loc) ? NULL : &equation[loc1-1].token.operatr, &d1, op, d2)) {
 				if (op == POWER && !domain_check) {
 					if (!f_to_fraction(d2, &numerator, &denominator)) {	/* if irrational power */
-						if (!iflag || (preserve_roots && !approximate_roots))
+						if (!iflag || (preserve_surds && !approximate_roots))
 							return modified;
-						cv.re = d;
+						cv.re = d1;
 						cv.im = 0.0;
 						p.re = d2;
 						p.im = 0.0;
@@ -1142,13 +1193,13 @@ beginning:
 						return true;
 					}
 					errno = 0;
-					d1 = pow(-d, d2);
+					d3 = pow(-d1, d2);
 					check_err();
 					if (!always_positive(denominator)) {
 						if (!always_positive(numerator)) {
-							d1 = -d1;
+							d3 = -d3;
 						}
-						d = d1;
+						d1 = d3;
 						goto not_imaginary;
 					}
 					if (!iflag)
@@ -1161,7 +1212,7 @@ beginning:
 					if (d2 == 0.5) {
 						equation[loc1].level = level + 1;
 						equation[loc1].kind = CONSTANT;
-						equation[loc1].token.constant = -d;
+						equation[loc1].token.constant = -d1;
 						loc1++;
 						equation[loc1].level = level + 1;
 						equation[loc1].kind = OPERATOR;
@@ -1180,7 +1231,7 @@ beginning:
 					} else {
 						equation[loc1].level = level;
 						equation[loc1].kind = CONSTANT;
-						equation[loc1].token.constant = d1;
+						equation[loc1].token.constant = d3;
 						loc1++;
 						equation[loc1].level = level;
 						equation[loc1].kind = OPERATOR;
@@ -1201,7 +1252,7 @@ beginning:
 					return true;
 				} else {
 not_imaginary:
-					equation[loc1].token.constant = d;
+					equation[loc1].token.constant = d1;
 					modified = true;
 					domain_check = false;
 					blt(&equation[loc-1], &equation[loc+1], (*np - (loc + 1)) * sizeof(token_type));
@@ -1536,6 +1587,8 @@ int		n2, l2, *diff_signp;
 				return true;
 			}
 			break;
+		case OPERATOR:
+			break;
 		}
 		return false;
 	}
@@ -1615,6 +1668,7 @@ no_op2:
 			opa2[oc2] = pv1 + 1;
 			used[oc2] = false;
 			if (++oc2 >= ARR_CNT(opa2)) {
+				debug_string(1, "Expression too big to compare, because MAX_COMPARE_TERMS may be too small.");
 				return false;	/* expression too big to compare */
 			}
 		}
@@ -2097,16 +2151,17 @@ int		*np, loc, level;
 	}
 	return modified;
 corrupt:
-	error(_("Expression is corrupt!  Please file a bug report."));
-	longjmp(jmp_save, 13);
+	error_bug("Expression is corrupt!  Please send a bug report.");
+	return modified;	/* not reached */
 }
 
 /*
- * Rationalize the denominator of algebraic fractions.
- * Only works with square roots.
+ * Try to rationalize the denominator of algebraic fractions.
+ * Only works with a square root in the denominator
+ * and sometimes helps with simplification.
  *
  * Returns true if something was done.
- * Unfactoring needs to be done immediately, if it returns true.
+ * Unfactoring needs to be done immediately, if this returns true.
  */
 int
 rationalize(equation, np)
@@ -2171,13 +2226,13 @@ do_again:
 					    && equation[k].level == equation[k+1].level
 					    && equation[k+1].kind == CONSTANT
 					    && fmod(equation[k+1].token.constant, 1.0) == 0.5) {
-						if (count != 1) {
-							for (k1 = i + 2; k1 < end_loc; k1 += 2) {
-								if (equation[k1].token.operatr == POWER
-								    && equation[k1].level == equation[k1+1].level
-								    && equation[k1+1].kind == CONSTANT
-								    && fmod(equation[k1+1].token.constant, 1.0) == 0.5
-								    && k != k1) {
+						for (k1 = i + 2; k1 < end_loc; k1 += 2) {
+							if (equation[k1].token.operatr == POWER
+							    && equation[k1].level == equation[k1+1].level
+							    && equation[k1+1].kind == CONSTANT
+							    && fmod(equation[k1+1].token.constant, 1.0) == 0.5
+							    && k != k1) {
+								if (!(equation[k1].level == (div_level + 2) && count == 1)) {
 									i += 2;
 									goto be_thorough;
 								}
@@ -2215,8 +2270,9 @@ do_again:
 						equation[k1].token.operatr = DIVIDE;
 						k1++;
 						blt(&equation[k1], scratch, k * sizeof(token_type));
-						modified = true;
 						i = k1 + k;
+						debug_string(1, "Square roots in denominator rationalized.");
+						modified = true;
 						goto be_thorough;
 					}
 				}
