@@ -1,7 +1,7 @@
 /*
  * Mathomatic symbolic solve routines.
  *
- * Copyright (C) 1987-2008 George Gesslein II.
+ * Copyright (C) 1987-2009 George Gesslein II.
  */
 
 #include "includes.h"
@@ -64,7 +64,7 @@ int	have;	/* equation number to solve */
  * Returns 1 for normal success.
  * Returns 2 if successful and a solution was zero and removed.
  * Returns 0 on failure.  Might succeed at a numeric solve.
- * Returns -1 if the equation is an identity.
+ * Returns -1 if solving for a variable and the equation is an identity.
  * Returns -2 if unsolvable in all realms.
  */
 int
@@ -104,7 +104,7 @@ int		*rightnp;	/* pointer to length of RHS */
 		return false;
 	}
 	if (n_tlhs <= 0 || n_trhs <= 0) {
-		error(_("Please enter an equation or type \"help\"."));
+		error(_("Please enter an equation or a command like \"help\"."));
 		return false;
 	}
 	if (wantp->kind == VARIABLE) {
@@ -123,8 +123,10 @@ int		*rightnp;	/* pointer to length of RHS */
 		}
 		zsolve = true;
 	}
+#if	true
 	uf_power(tlhs, &n_tlhs);
 	uf_power(trhs, &n_trhs);
+#endif
 simp_again:
 	/* Make sure equation is a bit simplified. */
 	list_tdebug(2);
@@ -465,9 +467,9 @@ zero_simp:
 						qtries = 0;	/* might be quadratic after removing solution */
 						success = 2;
 #if	!SILENT
-						printf(_("Removing possible solution: \""));
+						fprintf(gfp, _("Removing possible solution: \""));
 						list_proc(b1, 1, false);
-						printf(" = 0\".\n");
+						fprintf(gfp, " = 0\".\n");
 #endif
 					} else {
 						debug_string(1, _("Juggling..."));
@@ -506,12 +508,11 @@ zero_simp:
 					if (inc_count > MAX_RAISE_POWER)
 						return false;
 #if	!SILENT
-					printf(_("Raising both sides to the power of %g and unfactoring...\n"), 1.0 / b1->token.constant);
+					fprintf(gfp, _("Raising both equation sides to the power of %.*g and unfactoring...\n"), precision, 1.0 / b1->token.constant);
 #endif
 					zero_solved = false;
 					qtries = 0;
 					if (!increase(b1->token.constant, v)) {
-						error(_("Failed to raise both sides to a power."));
 						return false;
 					}
 					uf_flag = true;
@@ -571,15 +572,17 @@ long	v;
 	partial_flag = false;
 	ufactor(tlhs, &n_tlhs);
 	partial_flag = true;
+	symb_flag = symblify;
 	simp_ssub(tlhs, &n_tlhs, v, d, true, false, 2);
-	simp_ssub(tlhs, &n_tlhs, 0L, 0.0, true, true, 2);
+	simp_ssub(tlhs, &n_tlhs, 0L, 1.0, true, true, 2);
+	symb_flag = false;
 lonely:
 	ep = &tlhs[n_tlhs];
 	len2 = len1 = 0;
 	foundp = false;
 	for (p1 = tlhs + 1;; p1 += 2) {
 		if (p1 >= ep) {
-			return false;
+			return true;	/* power not found */
 		}
 		if (p1->level == 1) {
 			break;
@@ -782,7 +785,7 @@ long	v;	/* solve variable */
 		return false;
 #if	!SILENT
 	list_var(v, 0);
-	printf(_("Equation is a degree %.*g polynomial in %s.\n"), precision, high_power, var_str);
+	fprintf(gfp, _("Equation is a degree %.*g polynomial in (%s).\n"), precision, high_power, var_str);
 #endif
 	if (a1p > trhs && (a1p - 1)->token.operatr == MINUS)
 		opx1 = MINUS;
@@ -989,16 +992,16 @@ big_bbreak:
 	n_trhs = len;
 	simp_loop(trhs, &n_trhs);
 	list_tdebug(2);
-	uf_simp(trhs, &n_trhs);
+	uf_tsimp(trhs, &n_trhs);	/* don't unfactor result so much, just unfactor what will be unfactored anyway */
 	simps_side(trhs, &n_trhs, false);
 	list_tdebug(1);
 #if	!SILENT
 	if (high_power == 2.0) {
-		printf(_("Equation was quadratic.\n"));
+		fprintf(gfp, _("Equation was quadratic.\n"));
 	} else if (high_power == 4.0) {
-		printf(_("Equation was biquadratic.\n"));
+		fprintf(gfp, _("Equation was biquadratic.\n"));
 	} else {
-		printf(_("Equation was solved with the quadratic formula.\n"));
+		fprintf(gfp, _("Equation was solved with the quadratic formula.\n"));
 	}
 #endif
 	return true;
@@ -1071,15 +1074,15 @@ int		*side2np;	/* pointer to the length of "side2p" */
 			return false;
 		if (!get_constant(operandp, operandn, &d2))
 			return false;
-		debug_string(1, _("Taking logarithm of both sides:"));
+		debug_string(1, _("Taking logarithm of both equation sides:"));
 		*side2np = 1;
 		side2p->level = 1;
 		side2p->kind = CONSTANT;
 		errno = 0;
 		side2p->token.constant = log(d1) / log(d2);
-		check_err();
 		blt(side1p, p1 + 1, (*side1np - (operandn + 1)) * sizeof(token_type));
 		*side1np -= operandn + 1;
+		check_err();
 		return true;
 	}
 	if (op == MODULUS) {
@@ -1094,52 +1097,60 @@ int		*side2np;	/* pointer to the length of "side2p" */
 	if (debug_level > 0) {
 		switch (op) {
 		case PLUS:
-			printf(_("Subtracting"));
+			fprintf(gfp, _("Subtracting"));
 			break;
 		case MINUS:
-			printf(_("Adding"));
+			fprintf(gfp, _("Adding"));
 			break;
 		case TIMES:
-			printf(_("Dividing both sides of the equation by"));
+			fprintf(gfp, _("Dividing both sides of the equation by"));
 			break;
 		case DIVIDE:
-			printf(_("Multiplying both sides of the equation by"));
+			fprintf(gfp, _("Multiplying both sides of the equation by"));
 			break;
 		case POWER:
-			printf(_("Raising both sides of the equation to the power of"));
+			fprintf(gfp, _("Raising both sides of the equation to the power of"));
 			break;
 		case MODULUS:
-			printf(_("Applying inverse modulus of"));
+			fprintf(gfp, _("Applying inverse modulus of"));
 			break;
 		}
-		printf(" \"");
-		if (op == POWER)
-			printf("1/(");
-		list_proc(operandp, operandn, false);
-		switch (op) {
-		case PLUS:
-			printf(_("\" from both sides of the equation:\n"));
-			break;
-		case MINUS:
-		case MODULUS:
-			printf(_("\" to both sides of the equation:\n"));
-			break;
-		case POWER:
-			printf(")");
-		default:
-			printf("\":\n");
-			break;
+		if (op == POWER && operandn == 1 && operandp->kind == CONSTANT) {
+			fprintf(gfp, " %.*g:\n", precision, 1.0 / operandp->token.constant);
+		} else {
+			fprintf(gfp, " \"");
+			if (op == POWER)
+				fprintf(gfp, "1/(");
+			list_proc(operandp, operandn, false);
+			switch (op) {
+			case PLUS:
+				fprintf(gfp, _("\" from both sides of the equation:\n"));
+				break;
+			case MINUS:
+			case MODULUS:
+				fprintf(gfp, _("\" to both sides of the equation:\n"));
+				break;
+			case POWER:
+				fprintf(gfp, ")");
+			default:
+				fprintf(gfp, "\":\n");
+				break;
+			}
 		}
 	}
 #endif
 	if (*side1np + operandn + 3 > n_tokens || *side2np + operandn + 5 > n_tokens) {
 		error_huge();
 	}
-	for (p2 = side1p; p2 < ep; p2++)
-		p2->level++;
+	if (min_level(side1p, oldn) <= 1) {
+		for (p2 = side1p; p2 < ep; p2++)
+			p2->level++;
+	}
 	ep = &side2p[*side2np];
-	for (p2 = side2p; p2 < ep; p2++)
-		p2->level++;
+	if (min_level(side2p, *side2np) <= 1) {
+		for (p2 = side2p; p2 < ep; p2++)
+			p2->level++;
+	}
 	p2 = &side1p[oldn];
 	switch (op) {
 	case MODULUS:
@@ -1149,7 +1160,7 @@ int		*side2np;	/* pointer to the length of "side2p" */
 		p2++;
 		p2->level = 2;
 		p2->kind = VARIABLE;
-		parse_var(&p2->token.variable, "integer");
+		parse_var(&p2->token.variable, V_INTEGER_NAME);
 		p2++;
 		p2->level = 2;
 		p2->kind = OPERATOR;
